/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ConfigFoldersPage.cpp
*
* Purpose : Configuration Dialog - Folders Page
*
* Authors : Anuj Seth
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "YahooPOPs.h"
#include "ConfigFoldersPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CDataHolder settings;

IMPLEMENT_DYNCREATE(CConfigFoldersPage, CPropertyPage)

CConfigFoldersPage::CConfigFoldersPage() : CCustomPropertyPage(CConfigFoldersPage::IDD)
{
	//{{AFX_DATA_INIT(CConfigFoldersPage)
	m_strSelectedFolder = _T("");
	m_strNewFolder = _T("");
	//}}AFX_DATA_INIT
}

CConfigFoldersPage::~CConfigFoldersPage()
{
}

void CConfigFoldersPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigFoldersPage)
	DDX_Control(pDX, IDC_REMOVE, m_btnRemove);
	DDX_Control(pDX, IDC_ADD, m_btnAdd);
	DDX_Control(pDX, IDC_FOLDERS, m_lstFolders);
	DDX_LBString(pDX, IDC_FOLDERS, m_strSelectedFolder);
	DDX_Text(pDX, IDC_NEW_FOLDER, m_strNewFolder);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigFoldersPage, CPropertyPage)
	//{{AFX_MSG_MAP(CConfigFoldersPage)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigFoldersPage message handlers

void CConfigFoldersPage::Init(CDataHolder &data)
{
	int prevLoc = 0;

	while(1)
	{
		int i = settings.m_strFolderList.Find("|", prevLoc);

		if(i == -1)
		{
			CStdString folderName = settings.m_strFolderList.Mid(prevLoc);

			if(!folderName.IsEmpty())
			{
				m_lstFolders.AddString(folderName);
			}

			break;
		}
		
		CStdString folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
		m_lstFolders.AddString(folderName);
		prevLoc = i + 1;
	}

	UpdateData(FALSE);
}

BOOL CConfigFoldersPage::ValidateData(void)
{
	UpdateData(TRUE);

	CString folderName;
	settings.m_strFolderList = "";

	for(int i = 0; i < m_lstFolders.GetCount(); i++)
	{
		m_lstFolders.GetText(i, folderName);

		if(!settings.m_strFolderList.IsEmpty())
		{
			settings.m_strFolderList += "|";
		}

		settings.m_strFolderList += folderName;
	}

	return TRUE;
}

void CConfigFoldersPage::SaveData(CDataHolder &data)
{
	CString folderName;
	CString folderList = "";

	for(int i = 0; i < m_lstFolders.GetCount(); i++)
	{
		m_lstFolders.GetText(i, folderName);

		if(!folderList.IsEmpty())
		{
			folderList += "|";
		}

		folderList += folderName;
	}

	settings.m_strFolderList = folderList;
	data.m_strFolderList = folderList;
}

void CConfigFoldersPage::OnAdd() 
{
	CString str;
	bool bExists = false;

	UpdateData(TRUE);

	for(int i = 0; i < m_lstFolders.GetCount(); i++)
	{
		m_lstFolders.GetText(i, str);

		if(str == m_strNewFolder) 
		{
			bExists = true;
			break;
		}
	}

	if(!bExists)
	{
		m_lstFolders.AddString(m_strNewFolder);
		m_strNewFolder = "";
		UpdateData(FALSE);
	}
}

void CConfigFoldersPage::OnRemove() 
{
	int selected = 0;

	if((selected = m_lstFolders.GetCurSel()) != -1)
	{
		m_lstFolders.DeleteString(selected);
		UpdateData(FALSE);
	}
}

#endif // #ifdef WIN32
