/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ConfigProxyPage.cpp
*
* Purpose : Configuration Dialog - Proxy Page
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "YahooPOPs.h"
#include "ConfigProxyPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CDataHolder settings;

IMPLEMENT_DYNCREATE(CConfigProxyPage, CPropertyPage)

CConfigProxyPage::CConfigProxyPage() : CCustomPropertyPage(CConfigProxyPage::IDD)
{
	//{{AFX_DATA_INIT(CConfigProxyPage)
	m_bEnableProxyAuth = FALSE;
	m_strHttpProxyHost = _T("");
	m_nHttpProxyPort = 80;
	m_strHttpsProxyHost = _T("");
	m_nHttpsProxyPort = 80;
	m_bUseProxy = FALSE;
	m_strProxyAuthPassword = _T("");
	m_strProxyAuthUsername = _T("");
	//}}AFX_DATA_INIT
}

CConfigProxyPage::~CConfigProxyPage()
{
}

void CConfigProxyPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigProxyPage)
	DDX_Control(pDX, IDC_ENABLE_PROXY_AUTH, m_ctlEnableProxyAuth);
	DDX_Control(pDX, IDC_PROXY_AUTH_USER_NAME, m_ctlProxyAuthUsername);
	DDX_Control(pDX, IDC_PROXY_AUTH_PASSWORD, m_ctlProxyAuthPassword);
	DDX_Control(pDX, IDC_HTTP_PROXY_PORT, m_ctlHttpProxyPort);
	DDX_Control(pDX, IDC_HTTPS_PROXY_PORT, m_ctlHttpsProxyPort);
	DDX_Control(pDX, IDC_HTTPS_PROXY_HOST, m_ctlHttpsProxyHost);
	DDX_Control(pDX, IDC_HTTP_PROXY_HOST, m_ctlHttpProxyHost);
	DDX_Check(pDX, IDC_ENABLE_PROXY_AUTH, m_bEnableProxyAuth);
	DDX_Text(pDX, IDC_HTTP_PROXY_HOST, m_strHttpProxyHost);
	DDX_Text(pDX, IDC_HTTP_PROXY_PORT, m_nHttpProxyPort);
	DDV_MinMaxUInt(pDX, m_nHttpProxyPort, 1, 65535);
	DDX_Text(pDX, IDC_HTTPS_PROXY_HOST, m_strHttpsProxyHost);
	DDX_Text(pDX, IDC_HTTPS_PROXY_PORT, m_nHttpsProxyPort);
	DDV_MinMaxUInt(pDX, m_nHttpsProxyPort, 1, 65535);
	DDX_Check(pDX, IDC_USE_PROXY, m_bUseProxy);
	DDX_Text(pDX, IDC_PROXY_AUTH_PASSWORD, m_strProxyAuthPassword);
	DDX_Text(pDX, IDC_PROXY_AUTH_USER_NAME, m_strProxyAuthUsername);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigProxyPage, CCustomPropertyPage)
	//{{AFX_MSG_MAP(CConfigProxyPage)
	ON_BN_CLICKED(IDC_USE_PROXY, OnUseProxy)
	ON_BN_CLICKED(IDC_ENABLE_PROXY_AUTH, OnEnableProxyAuth)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CConfigProxyPage::Init(CDataHolder &data)
{
	m_bEnableProxyAuth = data.m_bProxyAuth;
	m_strProxyAuthPassword = data.m_strProxyAuthPassword;
	m_strProxyAuthUsername = data.m_strProxyAuthUsername;

	m_bUseProxy		   = data.m_bUseProxy;

	m_strHttpsProxyHost= data.m_strHttpsProxyHost;
	m_nHttpsProxyPort  = data.m_nHttpsProxyPort;

	m_strHttpProxyHost= data.m_strHttpProxyHost;
	m_nHttpProxyPort  = data.m_nHttpProxyPort;

	UpdateData(FALSE);

	OnUseProxy();
	OnEnableProxyAuth();
}

BOOL CConfigProxyPage::ValidateData(void)
{
	UpdateData(TRUE);

	m_strHttpProxyHost.TrimLeft();

	if(m_strHttpProxyHost.IsEmpty())
	{
		m_bUseProxy = FALSE;
	}

	m_strProxyAuthPassword.TrimRight();
	m_strProxyAuthUsername.TrimRight();

	if(m_strProxyAuthPassword.IsEmpty() && m_strProxyAuthUsername.IsEmpty())
	{
		m_bEnableProxyAuth = FALSE;
	}

	return TRUE;
}

void CConfigProxyPage::SaveData(CDataHolder &data)
{
	settings.m_bProxyAuth = m_bEnableProxyAuth;
	settings.m_strHttpProxyHost = m_strHttpProxyHost;
	settings.m_strHttpsProxyHost = m_strHttpsProxyHost;
	settings.m_nHttpProxyPort = m_nHttpProxyPort;
	settings.m_nHttpsProxyPort = m_nHttpsProxyPort;
	settings.m_bUseProxy = m_bUseProxy;
	settings.m_strProxyAuthPassword = m_strProxyAuthPassword;
	settings.m_strProxyAuthUsername = m_strProxyAuthUsername;

	data.m_bProxyAuth = m_bEnableProxyAuth;
	data.m_strHttpProxyHost = m_strHttpProxyHost;
	data.m_strHttpsProxyHost = m_strHttpsProxyHost;
	data.m_nHttpProxyPort = m_nHttpProxyPort;
	data.m_nHttpsProxyPort = m_nHttpsProxyPort;
	data.m_bUseProxy = m_bUseProxy;
	data.m_strProxyAuthPassword = m_strProxyAuthPassword;
	data.m_strProxyAuthUsername = m_strProxyAuthUsername;
}

void CConfigProxyPage::OnUseProxy() 
{
	UpdateData(TRUE);

	m_ctlHttpProxyHost.EnableWindow(m_bUseProxy);
	m_ctlHttpProxyPort.EnableWindow(m_bUseProxy);
	m_ctlHttpsProxyHost.EnableWindow(m_bUseProxy);
	m_ctlHttpsProxyPort.EnableWindow(m_bUseProxy);
	m_ctlEnableProxyAuth.EnableWindow(m_bUseProxy);

	if(m_bUseProxy == FALSE)
	{
		m_ctlProxyAuthPassword.EnableWindow(FALSE);
		m_ctlProxyAuthUsername.EnableWindow(FALSE);
	}
	else if(m_bEnableProxyAuth)
	{
		m_ctlProxyAuthPassword.EnableWindow(TRUE);
		m_ctlProxyAuthUsername.EnableWindow(TRUE);
	}
}

void CConfigProxyPage::OnEnableProxyAuth() 
{
	UpdateData(TRUE);

	if(!m_bUseProxy)
	{
		m_bEnableProxyAuth = FALSE;
		UpdateData(FALSE);
	}

	m_ctlProxyAuthPassword.EnableWindow(m_bEnableProxyAuth);
	m_ctlProxyAuthUsername.EnableWindow(m_bEnableProxyAuth);
}

#endif // #ifdef WIN32
