/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*                   http://www.anujseth.com/yahoopops
*
* Credits:
* - YahooPOPs! has been inspired by FetchYahoo, a Perl based application
*   that works with mail clients supporting mbox'es (Netscape, Pine, etc.).
*   FetchYahoo is authored by Ravi Ramkissoon <ravi_ramkissoon@yahoo.com>
*-------------------------------------------------------------------------
*
* Name    :	ConfigureDlg.cpp
*
* Purpose : Dialog to configure YahooPOPs!
*
* Author  : Anuj Seth <yahoopops@anujseth.com>
* 			K.Shyam
*
* License : GNU Public License
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "YahooPOPs.h"
#include "ConfigureDlg.h"
#include "DataHolder.h"

#ifdef _WIN32
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#endif

extern CDataHolder settings;

CConfigureDlg::CConfigureDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CConfigureDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CConfigureDlg)
	m_bSecureMode = FALSE;
	m_bUseProxy = FALSE;
	m_nUserAgent = -1;
	m_strHttpProxyHost = _T("");
	m_nHttpProxyPort = 80;
	m_strHttpsProxyHost = _T("");
	m_nHttpsProxyPort = 80;
	m_bUseMD5 = FALSE;
	m_nMaxEmails = 1000;
	//}}AFX_DATA_INIT
}


void CConfigureDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigureDlg)
	DDX_Control(pDX, IDC_USER_AGENT, m_ctlUserAgent);
	DDX_Control(pDX, IDC_HTTPS_PROXY_PORT, m_ctlHttpsProxyPort);
	DDX_Control(pDX, IDC_HTTPS_PROXY_HOST, m_ctlHttpsProxyHost);
	DDX_Control(pDX, IDC_HTTP_PROXY_PORT, m_ctlHttpProxyPort);
	DDX_Control(pDX, IDC_HTTP_PROXY_HOST, m_ctlHttpProxyHost);
	DDX_Check(pDX, IDC_HTTPS_MODE, m_bSecureMode);
	DDX_Check(pDX, IDC_USE_HTTP_PROXY, m_bUseProxy);
	DDX_CBIndex(pDX, IDC_USER_AGENT, m_nUserAgent);
	DDX_Text(pDX, IDC_HTTP_PROXY_HOST, m_strHttpProxyHost);
	DDX_Text(pDX, IDC_HTTP_PROXY_PORT, m_nHttpProxyPort);
	DDV_MinMaxUInt(pDX, m_nHttpProxyPort, 1, 65535);
	DDX_Text(pDX, IDC_HTTPS_PROXY_HOST, m_strHttpsProxyHost);
	DDX_Text(pDX, IDC_HTTPS_PROXY_PORT, m_nHttpsProxyPort);
	DDV_MinMaxUInt(pDX, m_nHttpsProxyPort, 1, 65535);
	DDX_Check(pDX, IDC_USE_MD5, m_bUseMD5);
	DDX_Text(pDX, IDC_MAX_EMAILS, m_nMaxEmails);
	DDV_MinMaxUInt(pDX, m_nMaxEmails, 1, 1000);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigureDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigureDlg)
	ON_BN_CLICKED(IDC_USE_HTTP_PROXY, OnUseHttpProxy)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*******************************************************************
* Function  : OnUseHttpProxy()
* Parameters: -
* Returns   : -
* Purpose   : Enable/Disable controls appropriately
*******************************************************************/
void CConfigureDlg::OnUseHttpProxy()
{
	UpdateData(TRUE);
	
	m_ctlHttpProxyHost.EnableWindow(m_bUseProxy);
	m_ctlHttpProxyPort.EnableWindow(m_bUseProxy);
	m_ctlHttpsProxyHost.EnableWindow(m_bUseProxy);
	m_ctlHttpsProxyPort.EnableWindow(m_bUseProxy);
}

/*******************************************************************
* Function  : OnInitDialog()
* Parameters: -
* Returns   : 
* Purpose   : If you know MFC, you know what this function does ;)
*******************************************************************/
BOOL CConfigureDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_bSecureMode		= settings.m_bSecureMode;
	m_strHttpProxyHost	= settings.m_strHttpProxyHost;
	m_nHttpProxyPort	= settings.m_nHttpProxyPort;
	m_strHttpsProxyHost	= settings.m_strHttpsProxyHost;
	m_nHttpsProxyPort	= settings.m_nHttpsProxyPort;
	m_bUseProxy			= settings.m_bUseProxy;
	m_nUserAgent		= settings.m_nUserAgent;
	m_bUseMD5			= settings.m_bUseMD5;
	m_nMaxEmails		= settings.m_nMaxEmails;

	UpdateData(FALSE);

	OnUseHttpProxy();

	return TRUE;
}

/*******************************************************************
* Function  : OnOK()
* Parameters: 
* Returns   :
* Purpose   : If you know MFC, you know what this function does ;)
*******************************************************************/
void CConfigureDlg::OnOK()
{
	UpdateData(TRUE);

	settings.m_bSecureMode		= m_bSecureMode;
	settings.m_strHttpProxyHost = m_strHttpProxyHost;
	settings.m_nHttpProxyPort	= m_nHttpProxyPort;
	settings.m_strHttpsProxyHost= m_strHttpsProxyHost;
	settings.m_nHttpsProxyPort	= m_nHttpsProxyPort;
	settings.m_bUseProxy		= m_bUseProxy;
	settings.m_nUserAgent		= m_nUserAgent;
	settings.m_bUseMD5          = m_bUseMD5;
	settings.m_nMaxEmails		= m_nMaxEmails;

	CString data;
	m_ctlUserAgent.GetLBText(m_nUserAgent, data);

	settings.m_strUserAgent = LPCTSTR(data);

	settings.Save();

	CDialog::OnOK();
}

#endif // #ifdef WIN32
