/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	DataHolder.cpp
*
* Purpose : Retrieve/Save the configured settings
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifndef WIN32
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#endif

#include "DataHolder.h"
#include "LogFile.h"
#include "YAHOOPOPS.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef WIN32
extern CSystemTrayApp theApp;
#endif // WIN32
extern LogFile logFile;

/*******************************************************************
* Function  : CDataHolder constructor
* Parameters: 
* Returns   :
* Purpose   : Initialize the object with defaults
*******************************************************************/
CDataHolder::CDataHolder()
{
#ifdef WIN32
	m_strUserAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90)";
#else
	strcpy(m_strUserAgent,"Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90)");
#endif
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   :
* Purpose   :
*******************************************************************/
CDataHolder::~CDataHolder()
{
}

/*******************************************************************
* Function  : Save()
* Parameters: -
* Returns   : -
* Purpose   : To save the configured settings to the registry
*******************************************************************/
void CDataHolder::Save()
{
#ifdef WIN32
	theApp.WriteProfileInt("Settings", "UseMD5", m_bUseMD5);
	theApp.WriteProfileInt("Settings", "SecureMode", m_bSecureMode);
	theApp.WriteProfileInt("Settings", "PlainMode", m_bPlain);
	theApp.WriteProfileInt("Settings", "HttpProxyPort", m_nHttpProxyPort);
	theApp.WriteProfileInt("Settings", "HttpsProxyPort", m_nHttpsProxyPort);
	theApp.WriteProfileInt("Settings", "UseProxy", m_bUseProxy);
	theApp.WriteProfileInt("Settings", "nUserAgent", m_nUserAgent);
	theApp.WriteProfileString("Settings", "HttpProxyHost", m_strHttpProxyHost);
	theApp.WriteProfileString("Settings", "HttpsProxyHost", m_strHttpsProxyHost);
	theApp.WriteProfileString("Settings", "UserAgent", m_strUserAgent);
	theApp.WriteProfileInt("Settings", "MaxEmails", m_nMaxEmails);
	theApp.WriteProfileInt("Settings", "EmptyTrash", m_bEmptyTrash);
	theApp.WriteProfileInt("Settings", "EmptyBulkMail", m_bEmptyBulkMail);
	theApp.WriteProfileInt("Settings", "BulkMail", m_bBulkMail);
	theApp.WriteProfileInt("Settings", "Inbox", m_bInbox);
	theApp.WriteProfileInt("Settings", "EmailCategory", m_nEmailCategory);
	theApp.WriteProfileInt("Settings", "ProxyAuth", m_bProxyAuth);
	theApp.WriteProfileString("Settings", "ProxyAuthPassword", m_strProxyAuthPassword);
	theApp.WriteProfileString("Settings", "ProxyAuthUsername", m_strProxyAuthUsername);
	theApp.WriteProfileInt("Settings", "QuietMode", m_bQuietMode);
	theApp.WriteProfileString("Settings", "NetworkIP", m_strNetworkIPAddr);
	theApp.WriteProfileInt("Settings", "NetworkPort", m_nNetworkPort);
	theApp.WriteProfileInt("Settings", "LeaveUnread", m_bLeaveUnread);
	theApp.WriteProfileInt("Settings", "AutoStartWithWindows", m_bAutoStartWithWindows);
	theApp.WriteProfileInt("Settings", "SmtpPort", m_nSmtpPort);
	theApp.WriteProfileString("Settings", "Folders", m_strFolderList);
    theApp.WriteProfileInt("Settings", "AlwaysOnTop", m_bAlwaysOnTop);
    theApp.WriteProfileInt("Settings", "EnableSmtp", m_bEnableSmtp);
    theApp.WriteProfileInt("Settings", "SaveEmailCopy", m_bSaveEmailCopy);
    theApp.WriteProfileInt("Settings", "EnableUIDL", m_bEnableUIDL);
    theApp.WriteProfileInt("Settings", "IgnoreExactSize", m_bIgnoreSize);
    theApp.WriteProfileInt("Settings", "HideTrayIcon", m_bHideTrayIcon);
	theApp.WriteProfileInt("Settings", "CookieTimeout", m_nCookieTimeout);

	// Activity Log
	theApp.WriteProfileInt("Settings", "CreateLog", m_bCreateLog);
	theApp.WriteProfileInt("Settings", "LimitLog", m_bLimitLog);
	theApp.WriteProfileInt("Settings", "LogLevel", m_nLogLevel);
	theApp.WriteProfileInt("Settings", "LogSize", m_nLogSize);
#endif
}

/*******************************************************************
* Function  : Load()
* Parameters: -
* Returns   : -
* Purpose   : To load the configured settings from the registry
*******************************************************************/
int CDataHolder::Load()
{
#ifdef WIN32
	m_bUseMD5			= theApp.GetProfileInt("Settings", "UseMD5", FALSE);
	m_bSecureMode		= theApp.GetProfileInt("Settings", "SecureMode", FALSE);
	m_bPlain    		= theApp.GetProfileInt("Settings", "PlainMode", FALSE);
	m_nHttpProxyPort	= theApp.GetProfileInt("Settings", "HttpProxyPort", 80);
	m_nHttpsProxyPort	= theApp.GetProfileInt("Settings", "HttpsProxyPort", 80);
	m_bUseProxy			= theApp.GetProfileInt("Settings", "UseProxy", FALSE);
	m_nUserAgent		= theApp.GetProfileInt("Settings", "nUserAgent", 0);
	m_strHttpProxyHost	= theApp.GetProfileString("Settings", "HttpProxyHost", NULL);
	m_strHttpsProxyHost	= theApp.GetProfileString("Settings", "HttpsProxyHost", NULL);
	m_strUserAgent      = theApp.GetProfileString("Settings", "UserAgent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows 98) YahooPops/0.3");
	m_nMaxEmails		= theApp.GetProfileInt("Settings", "MaxEmails", 50);
	m_bEmptyTrash		= theApp.GetProfileInt("Settings", "EmptyTrash", 0);
	m_bEmptyBulkMail	= theApp.GetProfileInt("Settings", "EmptyBulkMail", 0);
	m_bBulkMail			= theApp.GetProfileInt("Settings", "BulkMail", FALSE);
	m_bInbox			= theApp.GetProfileInt("Settings", "Inbox", TRUE);
	m_nEmailCategory	= (EMAIL_CATEGORY)theApp.GetProfileInt("Settings", "EmailCategory", EC_ALL);
	m_bProxyAuth		= theApp.GetProfileInt("Settings", "ProxyAuth", FALSE);
	m_strProxyAuthPassword = theApp.GetProfileString("Settings", "ProxyAuthPassword", "");
	m_strProxyAuthUsername = theApp.GetProfileString("Settings", "ProxyAuthUsername", "");
	m_bQuietMode		= theApp.GetProfileInt("Settings", "QuietMode", FALSE);
	m_strNetworkIPAddr  = theApp.GetProfileString("Settings", "NetworkIP", "127.0.0.1");
	m_nNetworkPort		= theApp.GetProfileInt("Settings", "NetworkPort", 110);
	m_bLeaveUnread		= theApp.GetProfileInt("Settings", "LeaveUnread", FALSE);
	m_bAutoStartWithWindows = theApp.GetProfileInt("Settings", "AutoStartWithWindows", FALSE);
	m_nSmtpPort			= theApp.GetProfileInt("Settings", "SmtpPort", 25);
	m_strFolderList     = theApp.GetProfileString("Settings", "Folders", "");
    m_bAlwaysOnTop      = theApp.GetProfileInt("Settings", "AlwaysOnTop", FALSE);
    m_bEnableSmtp       = theApp.GetProfileInt("Settings", "EnableSmtp", FALSE);
    m_bSaveEmailCopy    = theApp.GetProfileInt("Settings", "SaveEmailCopy", FALSE);
    m_bEnableUIDL       = theApp.GetProfileInt("Settings", "EnableUIDL", TRUE);
    m_bIgnoreSize       = theApp.GetProfileInt("Settings", "IgnoreSize", TRUE);
    m_bHideTrayIcon       = theApp.GetProfileInt("Settings", "HideTrayIcon", FALSE);
	m_nCookieTimeout    = theApp.GetProfileInt("Settings", "CookieTimeout", 24000);

	// Activity Log
	m_bCreateLog = theApp.GetProfileInt("Settings", "CreateLog", FALSE);
	m_bLimitLog  = theApp.GetProfileInt("Settings", "LimitLog", TRUE);
	m_nLogLevel  = theApp.GetProfileInt("Settings", "LogLevel", 0);
	m_nLogSize   = theApp.GetProfileInt("Settings", "LogSize", 50);

#else
	char filename[512];
	FILE *fp=NULL;
	char temp_str[1024];
	/* the define CONFIG_FILE_LOC comes from the makefile which inturn comes from the user :-) */
	sprintf(filename,"%s/%s",CONFIG_FILE_LOC,"ypopsrc");	
	if ( ( fp = fopen(filename,"r") ) == NULL )
	{
		fprintf(stderr,"The necessary preferences file was not found.Please copy the sample preferences file ypops_samplerc given along with the distribution to %s as ypopsrc\n",CONFIG_FILE_LOC);
		return(0);
	}
	/* first the inbox setting */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bInbox = atoi(temp_str);
	/* now the bulk e-mail setting*/
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bBulkMail = atoi(temp_str);
	/* emptying trash on exit */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bEmptyTrash = atoi(temp_str);
	/* leave messages unread */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bLeaveUnread = atoi(temp_str);
	/*maximum number of e-mails */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nMaxEmails = atoi(temp_str);
	if ( m_nMaxEmails > 50 ) m_nMaxEmails = 50;
	/*downloading all e-mails */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	if ( ( strcmp(temp_str,"a") ) == 0 )
	{
		m_nEmailCategory = EC_ALL;
	}
	else
	if ( ( strcmp(temp_str,"u") ) == 0 )
	{
		m_nEmailCategory = EC_UNREAD;
	}
	else
	if ( ( strcmp(temp_str,"f") ) == 0 )
	{
		m_nEmailCategory = EC_FLAGGED;
	}
	/* connection on secure mode */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bSecureMode = atoi(temp_str);
	/*password using MD5 */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bUseMD5 = atoi(temp_str);
	/*plain mode connection */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bPlain = atoi(temp_str);
	/*using proxy server ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bUseProxy = atoi(temp_str);
	/*http proxy server address */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	strcpy(m_strHttpProxyHost,temp_str);
	/*http proxy server port number */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nHttpProxyPort = atoi(temp_str);
	/*secure proxy server address */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	strcpy(m_strHttpsProxyHost,temp_str);
	/* secure proxy server port number */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nHttpsProxyPort = atoi(temp_str);
	/* use proxy authentication ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bProxyAuth = atoi(temp_str);
	/* proxy server username */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	strcpy(m_strProxyAuthUsername,temp_str);
	/* proxy server password */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	strcpy(m_strProxyAuthPassword,temp_str);
	/* the port we must run on */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nNetworkPort = atoi(temp_str);
	/* do we need log file generation ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bCreateLog = atoi(temp_str);
	/* basic logs ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nLogLevel = atoi(temp_str);
	/* advanced logs ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nLogLevel = atoi(temp_str);
	/* do we need to limit log file size ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bLimitLog = atoi(temp_str);
	/* maximum log file size */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nLogSize = atoi(temp_str);
	/* whether classical of beta interface ? */
/*	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nYMInterface = atoi(temp_str);*/
	/* the smtp port where we have to listen for connections */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nSmtpPort = atoi(temp_str);
	/* whether we need smtp or not */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bEnableSmtp = atoi(temp_str);
	/* whether to keep mail in yahoo sent mail folder */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bSaveEmailCopy = atoi(temp_str);
	/* whether to enable UIDL */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
 	m_bEnableUIDL = atoi(temp_str);
	/* whether to ignore exact size */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
 	m_bIgnoreSize = atoi(temp_str);
	/* cookie timeout value */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
 	m_nCookieTimeout = atoi(temp_str);
	/* the list of all folders seperated by a pipe | */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strFolderList = temp_str;
	/* i think we are done for now.. 
	 * so return ! */
#endif
	if (m_bUseMD5 && m_bSecureMode)
	{
		logFile.Write(LOG_BASIC, "Warning: Both secure modes selected. Selecting MD5");
		m_bSecureMode = FALSE;
	}
	return(1);
}

/*******************************************************************
* Function  : DumpToLog()
* Parameters: -
* Returns   : -
* Purpose   : To write the configuration details to log file
*******************************************************************/
void CDataHolder::DumpToLog()
{
    logFile.Write(LOG_ADVANCED, "=========================================================================");
    logFile.Write(LOG_ADVANCED, "Configuration:");
    logFile.Write(LOG_ADVANCED, " Log Mode = %s", m_nLogLevel == 0 ? "Basic" : "Advanced");
    logFile.Write(LOG_ADVANCED, " MD5 = %d, Secure Mode = %d", m_bUseMD5, m_bSecureMode);

#ifdef WIN32 
    logFile.Write(LOG_ADVANCED, " Use Proxy = %d, HTTP Proxy = %s:%d, HTTPS Proxy = %s:%d", m_bUseProxy, (m_strHttpProxyHost.IsEmpty() ? "-" : LPCTSTR(m_strHttpProxyHost)),  m_nHttpProxyPort, (m_strHttpsProxyHost.IsEmpty() ? "-": LPCTSTR(m_strHttpsProxyHost)), m_nHttpsProxyPort); 
#else
    logFile.Write(LOG_ADVANCED, " Use Proxy = %d, HTTP Proxy = %s:%d, HTTPS Proxy = %s:%d", m_bUseProxy, ( (m_strHttpProxyHost[0] == '\0' ) ? "-" : LPCTSTR(m_strHttpProxyHost)),  m_nHttpProxyPort, ( (m_strHttpsProxyHost[0] =='\0') ? "-": LPCTSTR(m_strHttpsProxyHost)), m_nHttpsProxyPort); 
#endif
    logFile.Write(LOG_ADVANCED, " Max Emails = %d, Leave as unread = %d", m_nMaxEmails, m_bLeaveUnread);
    logFile.Write(LOG_ADVANCED, " Emtpy Trash = %d, Empty Bulk = %d", m_bEmptyTrash, m_bEmptyBulkMail);
    logFile.Write(LOG_ADVANCED, " Download Inbox = %d, Download BulkMail = %d", m_bInbox, m_bBulkMail);
    
    logFile.Write(LOG_ADVANCED, " Email Category = %s", 
        m_nEmailCategory == EC_ALL ? "All" : 
        m_nEmailCategory == EC_UNREAD ? "Unread" : "Flagged");

#ifdef WIN32
    logFile.Write(LOG_ADVANCED, " Proxy Auth = %d, Username = %s, Password = <HIDDEN>", m_bProxyAuth, (m_strProxyAuthUsername.IsEmpty() ? "-" : LPCTSTR(m_strProxyAuthUsername)));
#else
    logFile.Write(LOG_ADVANCED, " Proxy Auth = %d, Username = %s, Password = <HIDDEN>", m_bProxyAuth, ((m_strProxyAuthUsername[0] == '\0') ? "-" : LPCTSTR(m_strProxyAuthUsername)));
#endif
    
    logFile.Write(LOG_ADVANCED, " Quiet Mode = %d, Autostart = %d, Always on Top = %d", m_bQuietMode, m_bAutoStartWithWindows, m_bAlwaysOnTop);
    logFile.Write(LOG_ADVANCED, " Network IP = %s, POP3 Port = %d, SMTP Enable = %d, SMTP Port = %d", LPCTSTR(m_strNetworkIPAddr), m_nNetworkPort, m_bEnableSmtp, m_nSmtpPort);
    logFile.Write(LOG_ADVANCED, " Save Sent Email = %d, Enable UIDL = %d, Ignore Exact Size = %d, Hide Tray Icon = %d", m_bSaveEmailCopy, m_bEnableUIDL, m_bIgnoreSize, m_bHideTrayIcon);
	logFile.Write(LOG_ADVANCED, " Cookie Timeout = %d", m_nCookieTimeout);
    logFile.Write(LOG_ADVANCED, " User Agent = %s", LPCTSTR(m_strUserAgent));
    logFile.Write(LOG_ADVANCED, "=========================================================================");
}
