/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	Email.cpp
*
* Purpose : Class for handling emails
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"
#include "YAHOOPOPS.h"
#include <string.h>
#include "Email.h"
#include "LogFile.h"

extern LogFile logFile;

/*******************************************************************
* Function  : Email constructor
* Parameters: -
* Returns   : -
* Purpose   : Initialize the object
*******************************************************************/
Email::Email()
{
	SetDeleted(FALSE);
	bDownloaded=FALSE;
	bHeaderDownloaded=FALSE;

	emailHeader = "";
	emailBody   = "";
    msgId = "";
}

Email::~Email()
{
}

/*******************************************************************
* Function  : SetDeleted()
* Parameters: status - Is the email to be marked as deleted?
* Returns   : -
* Purpose   : To mark an email as deleted
*******************************************************************/
void Email::SetDeleted(BOOL status)
{
	bDeleted = status;
}

/*******************************************************************
* Function  : IsDeleted()
* Parameters: -
* Returns   : TRUE - Email is marked as deleted
*             FALSE - Email is not marked as deleted
* Purpose   : To return the status of the email delete flag
*******************************************************************/
BOOL Email::IsDeleted()
{
	return bDeleted;
}

/*******************************************************************
* Function  : SendMessage
* Parameters: sock - The socket into which the email is to be
*                    written
* Returns   : -
* Purpose   : To send the email to the POP3 client
*******************************************************************/
void Email::SendMessage(SOCKET &sock)
{
	CStdString ok;
	ok.Format("+OK %d octets\r\n", emailHeader.GetLength() + emailBody.GetLength() + 2);
	send(sock, ok, ok.GetLength(), 0);
	send(sock, emailHeader, emailHeader.GetLength(), 0);
	send(sock, emailBody, emailBody.GetLength(), 0);
	send(sock, "\r\n.\r\n", 5, 0);
}

/*******************************************************************
* Function  : SendMessage
* Parameters: sock - The socket into which the email is to be
*                    written
*             lines - The number of lines of the email body that is
*                    to be sent
* Returns   : -
* Purpose   : To send the email to the POP3 client
*******************************************************************/
void Email::SendMessage(SOCKET &sock, int lines)
{
	int i = 0;
	int startPos = 0;
	int pos = -1;

	for(i = 0; i < lines; i++)
	{
		pos = emailBody.Find("\r\n", startPos);

		if(pos == -1)
		{
			break;
		}
		else
		{
			startPos = pos + 1;
		}
	}

	if(pos == -1)
	{
		/* Send the entire message */
		CStdString ok;
		ok.Format("+OK %d octets\r\n", emailHeader.GetLength() + emailBody.GetLength() + 2);
		send(sock, ok, ok.GetLength(), 0);

		send(sock, emailHeader, emailHeader.GetLength(), 0);
		send(sock, emailBody, emailBody.GetLength(), 0);
	}
	else
	{
		CStdString ok;
		ok.Format("+OK %d octets\r\n", emailHeader.GetLength() + pos + 2);
		send(sock, ok, ok.GetLength(), 0);

		send(sock, emailHeader, emailHeader.GetLength(), 0);
		send(sock, emailBody, pos, 0);
	}

	send(sock, "\r\n.\r\n", 5, 0);
}

/*******************************************************************
* Function  : GetSize()
* Parameters: -
* Returns   : The size of the email (header + body)
* Purpose   : To return the size of the email. It is used for the
*             STAT, LIST and RETR commands.
*******************************************************************/
int Email::GetSize()
{
	if(IsDownloaded())
	{
		return (emailHeader.GetLength() + emailBody.GetLength());
	}
	else
	{
		return tempsize;
	}
}

/*******************************************************************
* Function  : IsDownloaded()
* Parameters: -
* Returns   : The status of the download flag
* Purpose   : To determine if the email has been downloaded. If for
*             some reason, a particular email could not be 
*             downloaded, and a RETR is done on it, YahooPOPs! will
*             return an error.
*******************************************************************/
BOOL Email::IsDownloaded()
{
	return bDownloaded;
}

/*******************************************************************
* Function  : IsDownloaded()
* Parameters: -
* Returns   : The status of the header download flag
* Purpose   : To determine if the email header has been downloaded. If for
*             some reason, a particular email header could not be 
*             downloaded, and a RETR is done on it, YahooPOPs! will
*             return an error.
*******************************************************************/
BOOL Email::IsHeaderDownloaded()
{
	return bHeaderDownloaded;
}

/*******************************************************************
* Function  : SetHeader()
* Parameters: data - The email headers
* Returns   : -
* Purpose   : To set the email object with its headers.
*******************************************************************/
void Email::SetHeader(CStdString data)
{
	/* 
	 * Check if the first line is something like:
	 * "From Anuj Seth Tue May 14 10:22:06 2002"
	 * (excluding the quotes). If this is there
	 * remove it!
	 */
	int endPos = 0;

	if(data.Find("From ") == 0)
	{
		endPos = data.Find("\n");

		if(endPos == -1)
		{
			/* Something's wrong! */
            logFile.Write(LOG_BASIC, "ERROR: Unable to set header. EOL not found!");
			return;
		}

		endPos++;
	}

	emailHeader = data.Mid(endPos);
	emailHeader.Replace("\n", "\r\n");

	DeleteHeader("X-RocketMail");
	DeleteHeader("X-RocketUID");
	DeleteHeader("X-RocketMIF");
	DeleteHeader("X-RocketRCL");
	DeleteHeader("X-Track");
	DeleteHeader("X-Rocket-Server");
	DeleteHeader("X-Rocket-Track");
	DeleteHeader("Content-Transfer-Encoding: quoted-printable");

	bHeaderDownloaded=TRUE;
}

/*******************************************************************
* Function  : GetHeader()
* Parameters: name - header to find
* Returns   : -
* Purpose   : To find the email header value
*******************************************************************/
CStdString Email::GetHeader(CStdString name)
{
	int pos;
	if (name=="") return "";

	name.MakeLower();
	CStdString headers=emailHeader;

	/* Find the header name */
	if ((pos=headers.ToLower().Find(name+":"))<0) return "";

	/* Make sure CRLF is right before the name
	   or this is the first header */
	if (pos!=0 && ((pos<2) || (emailHeader.Mid(pos-2,2)!="\r\n")))
		return "";

	/* Move pointer after the name, ":" and spaces */
	pos+=name.GetLength()+1;
	while (headers.GetAt(pos)==' ') pos++;

	/* find the end of the value string */
	for (int endPos=pos;;) {
		if ((endPos=headers.Find("\r\n",endPos))<0) return "";
		switch (headers.GetAt(endPos+2)) {
		/* if CRLF is followed by a white space, it is a folded
		   miltiline header */
		case ' ':
		case '\t':
		/* skip CRLF, white space and search again */
			endPos+=3;
			break;
		default:
			/* return a copy of the value string */
			return emailHeader.Mid(pos,endPos-pos);
		}
	}
}

/*******************************************************************
* Function  : SetBody()
* Parameters: data - The email body (including attachments)
* Returns   : -
* Purpose   : To set the email object with its body
*******************************************************************/
void Email::SetBody(CStdString data)
{
	emailBody = data;
	bDownloaded = TRUE;
}

/*******************************************************************
* Function  : AddHeader
* Parameters: name, value - The header to add to the email
* Returns   : TRUE - Header added
*             FALSE - Header not added
* Purpose   : To add a header to the email
*******************************************************************/
BOOL Email::AddHeader(CStdString name, CStdString value)
{
	if(name != NULL && value != NULL)
	{
		CStdString newHeader;

		newHeader += name;
		newHeader += ": ";
		newHeader += value;
		newHeader += "\r\n";
		newHeader += emailHeader;
		emailHeader = newHeader;

		return TRUE;
	}

	return FALSE;
}

/*******************************************************************
* Function  : DeleteHeader
* Parameters: name - The name of the header to delete
* Returns   : TRUE - header deleted
*             FALSE - header not deleted; possibly because the
*                    header does not exist
* Purpose   : To delete a specific header
*******************************************************************/
BOOL Email::DeleteHeader(CStdString name)
{
	CStdString str;
	int pos			= 0;
	int endPos		= 0;
	int startPos	= 0;
	CStdString headers=emailHeader;

	headers.ToLower();
	name.ToLower();
	// Fix provided by zerosum@users.sourceforge.net
	str.Format("%s: ", (LPCTSTR)name);

	while(true)
	{
		pos = headers.Find(name, startPos);

		if(pos == -1)
		{
			return FALSE;
		}

		/* 
		 * Let's confirm that the string is actually a
		 * header name and not part of some value!
		 */

		if( (pos == 0 && startPos == 0) || emailHeader[pos - 1] == '\r' || emailHeader[pos - 1] == '\n')
		{
			/* Found! Delete it! */
			endPos = emailHeader.Find("\r\n", pos + 1);
			endPos += 2;

			/* 
			 * Move the contents to the left, i.e.,
			 * delete one header
			 */

			emailHeader.Delete(pos, endPos - pos);
			return TRUE;
		}

		startPos++;
	}

	return FALSE;
}

const char *Email::GetMsgId(void)
{
    return (const char *)msgId;
}

void Email::SetMsgId(const CStdString &msgId)
{
    this->msgId = msgId;
}

void Email::SetTempSize(unsigned long size)
{
	tempsize=size;
}
