/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	HttpClient.h
*
* Purpose : A C++ class that provides with the basic HTTP GET and POST
*           methods. Proxy support is provided.
*
* Authors : Anuj Seth
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#ifndef _HTTP_CLIENT_H_
#define _HTTP_CLIENT_H_

#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

#include "StdString.h"
#include "Defines.h"
#include "HttpClientResponse.h"
#include "HttpSession.h"

#define ERROR_UNSUPPORTED_PROTOCOL -1
#define ERROR_SOCK_CREATION_FAILED -2
#define ERROR_CONNECT_FAILED       -3
#define HTTP_MAX_RETRIES            3

class HttpClient
{
public:
	// Methods
	int DoGet(const char *url, HttpClientResponse &resp);
	int SplitUrl(const char *url, char *host);
	BOOL DoPost(const char *url, const void * postData, HttpClientResponse &resp, bool formflag=false);

    // Headers
	BOOL AddHeader(const char *name, const char *value);

	// Proxy
	void UseHttpProxy(BOOL bUseProxy);
	void SetHttpProxy(const char *proxyHost, unsigned int proxyPort);
	void UseHttpsProxy(BOOL bUseProxy);
	void SetHttpsProxy(const char *proxyHost, unsigned int proxyPort);

	void Reset();
	void GetHost(CStdString &host);

	HttpClient(HttpSession *s);
	virtual ~HttpClient();

private:
   	int DoGetInternal(const char *url, HttpClientResponse &resp);
	BOOL DoPostInternal(const char *url, const void * postData, HttpClientResponse &resp, bool formflag=false);

protected:

	BOOL			bUseHttpProxy;
	char			httpProxyHost[256];
	unsigned int	httpProxyPort;

	BOOL			bUseHttpsProxy;
	char			httpsProxyHost[256];
	unsigned int	httpsProxyPort;

	char			*pResponsePage;
	unsigned int	responsePageLen;

	struct curl_slist *headers;
	CURL			*curl_handle;

	HttpSession		*session;
	char			host[256];
};

#endif

