/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	HttpClientResponse.cpp
*
* Purpose : A C++ class holds the response sent by the web server for a 
*           GET/POST method. This class is used by HttpClient.
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/

#include "stdafx.h"

#ifdef WIN32
#include <windows.h>
#endif

#include <iostream>
#include <string.h>
#include "HttpClientResponse.h"

/*******************************************************************
* Function  : HttpClientResponse constructor
* Parameters: -
* Returns   : -
* Purpose   : Initialize the object
*******************************************************************/
HttpClientResponse::HttpClientResponse()
{
	data		= NULL;
}

/*******************************************************************
* Function  : HttpClientResponse destructor
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
HttpClientResponse::~HttpClientResponse()
{
	if(data != NULL)
	{
		delete [] data;
	}
}

/*******************************************************************
* Function  : SetData()
* Parameters: response - The response data retrieved from the socket
*             responseLen - The length of the response data
* Returns   : -
* Purpose   : To set the response object with the data
*******************************************************************/
void HttpClientResponse::SetData(char *response, unsigned int responseLen)
{
	if(data != NULL) {
		delete [] data;
		data=NULL;
	}

	if(responseLen > 0)
	{
		dataLen = responseLen;
		data    = new char[dataLen + 1];
		memcpy(data, response, dataLen);
		data[dataLen]=0;
	}		
}


/*******************************************************************
* Function  : SetContentType()
* Parameters: contenttype - The content-type data
*             contenttypeLen - The length of the content-type data
* Returns   : -
* Purpose   : To set the response object with the data
*******************************************************************/
/* Edit SH. Keep track of the content type of responses */
void HttpClientResponse::SetContentType(char *contentType)
{
	strncpy(cdata, contentType, sizeof(cdata));
}

/*******************************************************************
* Function  : Reset()
* Parameters: -
* Returns   : -
* Purpose   : To partially cleanup the object so that it can be
*             reused for more requests.
*******************************************************************/
void HttpClientResponse::Reset()
{
	if(data != NULL)
	{
		delete [] data;
	}

	data		= NULL;
}

/*******************************************************************
* Function  : DataContains()
* Parameters: searchStr - The string to search for in the response
*                         data
* Returns   : TRUE - The string is present in the data
*             FALSE - The string is not present in the data
* Purpose   : To determine if the specified string is present
*             in the response data
*******************************************************************/
BOOL HttpClientResponse::DataContains(char *searchStr)
{
	if(data == NULL)
	{
		return FALSE;
	}

	if(strstr(data, searchStr) == NULL)
	{
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************
* Function  : ContentTypeContains()
* Parameters: searchStr - The string to search for in the ContentType
* Returns   : TRUE - The string is present in the cdata
*             FALSE - The string is not present in the cdata
* Purpose   : To determine if the specified string is present
*             in the content type
*******************************************************************/
BOOL HttpClientResponse::ContentTypeContains(char *searchStr)
{
	if(strstr(cdata, searchStr) == NULL)
	{
		return FALSE;
	}

	return TRUE;
}

/*******************************************************************
* Function  : GetBodyPtr()
* Parameters: -
* Returns   : Pointer to the HTTP body section
* Purpose   : To return a pointer to the HTTP body section so 
*             that it can be used for string manipulation, regular
*             expression search, etc. It is recommended that the
*             contents not be modified.
*******************************************************************/
char * HttpClientResponse::GetBodyPtr()
{
	return data;
}

/*******************************************************************
* Function  : GetBodyLength()
* Parameters: -
* Returns   : Size of the HTTP body section
* Purpose   : To return a size of the HTTP body section so 
*             that it can be used with the body pointer
*******************************************************************/
unsigned int HttpClientResponse::GetBodyLength() 
{
	return dataLen;
}
