/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	HttpSession.cpp
*
* Purpose : A C++ class that provides HTTP session (cookie-based) 
*           management. It is used by HttpClient
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/

#include "stdafx.h"
#include <iostream>
#include "HttpSession.h"
#include "HttpClient.h"
#include "HttpClientResponse.h"

/*******************************************************************
* Function  : HttpSession constructor
* Parameters: -
* Returns   : -
* Purpose   : Initialize the object
*******************************************************************/
HttpSession::HttpSession()
{
	bSessionMgmt = FALSE;
	cookieHead   = NULL;
	cookieTail   = NULL;
}

/*******************************************************************
* Function  : HttpSession destructor
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
HttpSession::~HttpSession()
{
	COOKIE *ptr = cookieHead;
	COOKIE *next = NULL;

	while(ptr != NULL)
	{
		next = ptr->next;
		delete ptr;
		ptr = next;
	}

	cookieHead = NULL;
	cookieTail = NULL;
}

/*******************************************************************
* Function  : SetSessionMgmt()
* Parameters: bEnabled - Are cookies to be saved?
* Returns   : -
* Purpose   : Enable/disable session management
*******************************************************************/
void HttpSession::SetSessionMgmt(BOOL bEnable)
{
	bSessionMgmt = bEnable;
}

/*******************************************************************
* Function  : SetCookies()
* Parameters: resp - The response object from which the cookies
*                    are to be extracted.
* Returns   : -
* Purpose   : Extract cookies
*******************************************************************/
void HttpSession::AddCookie(const char *value)
{
	if(bSessionMgmt == FALSE)
	{
		/* Session management is not required */
		return;
	}

	char cookie[4096];
	char *cookieStart = cookie;

	strcpy(cookie, value);

	char *ptr = strchr(cookie, ':');

	if(ptr != NULL)
	{
		while (*(++ptr) == ' ')
		{
			// Intentional empty loop
		}

		cookieStart = ptr;
	}

	/* 
	 * NOTE: The following can be a serious privacy issue.
	 * Since this class is currently being used only for
	 * YahooPOPs!, we are stripping out all the path
	 * information. So, this cookie will be sent for all
	 * requests to the Yahoo! mail server.
	 *
	 * TODO: Enhance session management to maintain a list
     * of cookies with relevant path information and send
	 * cookies for applicable domains only.
	 */

	ptr = strchr(cookieStart, ';');

	if(ptr != NULL)
	{
		*ptr = '\0';
	}

	ptr = strchr(cookieStart, '=');
		
	COOKIE *newCookie = new COOKIE;
	strncpy(newCookie->name, cookieStart, ptr - cookieStart);
	newCookie->name[ptr - cookieStart] = '\0';
	strcpy(newCookie->value, ptr + 1);
	newCookie->next = NULL;

	/* Check if this cookie already exists */
	COOKIE *cptr = cookieHead;

	while(cptr != NULL)
	{
		if(strcmp(cptr->name, newCookie->name) == 0)
		{
			/* This is not a new cookie! */
			strcpy(cptr->value, newCookie->value);
			delete newCookie;
			return;
		}

		cptr = cptr->next;
	}

	if(cookieHead == NULL)
	{
		cookieHead = newCookie;
		cookieTail = newCookie;
	}
	else
	{
		cookieTail->next = newCookie;
		cookieTail = newCookie;
	}
}

/*******************************************************************
* Function  : GetCookies()
* Parameters: req - The request object needs to be populated with
*                   the cookies for this session.
* Returns   : -
* Purpose   : Set cookies
*******************************************************************/
void HttpSession::GetCookies(HttpClient *req)
{
	if(bSessionMgmt == FALSE)
	{
		/* Session management is not required */
		return;
	}

	char cookies[10240];
	COOKIE *ptr = cookieHead;
	cookies[0] = '\0';

	while(ptr != NULL)
	{
		if(cookies[0] != '\0')
		{
			strcat(cookies, "; ");
		}

		strcat(cookies, ptr->name);
		strcat(cookies, "=");
		strcat(cookies, ptr->value);

		ptr = ptr->next;
	}

	if(cookies[0] != '\0')
	{
		req->AddHeader("Cookie", cookies);
	}
}

