/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	OptionsTreeDlg.cpp
*
* Purpose : Dialog to configure YahooPOPs!
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "OptionsTreeDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CDataHolder settings;
extern BOOL bConfigureVisible;

COptionsTreeDlg::COptionsTreeDlg(CWnd* pParent /*=NULL*/)
: CDialog(COptionsTreeDlg::IDD, pParent),
data(settings)
{
	m_bDlgCreated = false;
	
	//{{AFX_DATA_INIT(COptionsTreeDlg)
	// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void COptionsTreeDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionsTreeDlg)
	DDX_Control(pDX, IDC_TREE, m_Tree);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(COptionsTreeDlg, CDialog)
//{{AFX_MSG_MAP(COptionsTreeDlg)
ON_NOTIFY(TVN_SELCHANGING, IDC_TREE, OnSelchangingTree)
ON_NOTIFY(TVN_SELCHANGED, IDC_TREE, OnSelchangedTree)
ON_WM_DESTROY()
ON_WM_CONTEXTMENU()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL COptionsTreeDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	pInfoImages = new CImageList;
	pInfoImages->Create(16, 16, ILC_COLOR8 | ILC_MASK, 2, 0);
	pInfoImages->Add(AfxGetApp()->LoadIcon(IDI_CONFIG_ROOT));
	pInfoImages->Add(AfxGetApp()->LoadIcon(IDI_CONFIG_LEAF));
	m_Tree.SetImageList(pInfoImages, TVSIL_NORMAL);
	
	HTREEITEM hTreeEmailPref = m_Tree.InsertItem("Email Preferences", 0, 1, TVI_ROOT, TVI_LAST);
	m_Tree.SetItemData(hTreeEmailPref, (DWORD)&m_pageEmailPref);
	
	HTREEITEM hTreeEmail = m_Tree.InsertItem("Receiving Email", 0, 1, hTreeEmailPref, TVI_LAST);
	m_Tree.SetItemData(hTreeEmail, (DWORD)&m_pageEmail);

	HTREEITEM hTreeFolders = m_Tree.InsertItem("Download Folders", 0, 1, hTreeEmailPref, TVI_LAST);
	m_Tree.SetItemData(hTreeFolders, (DWORD)&m_pageFolders);

    HTREEITEM hTreeMailSmtp = m_Tree.InsertItem("Sending Email", 0, 1, hTreeEmailPref, TVI_LAST);
    m_Tree.SetItemData(hTreeMailSmtp, (DWORD)&m_pageMailSmtp);
	
	HTREEITEM hTreeAdvanced = m_Tree.InsertItem("Advanced Preferences", 0, 1, TVI_ROOT, TVI_LAST);
	m_Tree.SetItemData(hTreeAdvanced, (DWORD)&m_pageAdvanced);
	
	HTREEITEM hTreeSecurity = m_Tree.InsertItem("Security", 0, 1, hTreeAdvanced, TVI_LAST);
	m_Tree.SetItemData(hTreeSecurity, (DWORD)&m_pageSecurity);
	
	HTREEITEM hTreeProxy = m_Tree.InsertItem("Proxies", 0, 1, hTreeAdvanced, TVI_LAST);
	m_Tree.SetItemData(hTreeProxy, (DWORD)&m_pageProxy);
	
	HTREEITEM hTreeNetwork = m_Tree.InsertItem("Network", 0, 1, hTreeAdvanced, TVI_LAST);
	m_Tree.SetItemData(hTreeNetwork, (DWORD)&m_pageNetwork);

	HTREEITEM hTreeLog = m_Tree.InsertItem("Activity Log", 0, 1, hTreeAdvanced, TVI_LAST);
	m_Tree.SetItemData(hTreeLog, (DWORD)&m_pageLog);

	HTREEITEM hTreeMisc = m_Tree.InsertItem("Miscellaneous", 0, 1, TVI_ROOT, TVI_LAST);
	m_Tree.SetItemData(hTreeMisc, (DWORD)&m_pageMisc);
	
	m_pageAdvanced.Create(IDD_CONFIG_ADVANCED_PAGE, this);
	m_pageSecurity.Create(IDD_CONFIG_SECURITY_PAGE, this);
	m_pageProxy.Create(IDD_CONFIG_PROXIES_PAGE, this);
	m_pageEmailPref.Create(IDD_CONFIG_MAIL_PAGE, this);
	m_pageEmail.Create(IDD_CONFIG_EMAIL_PAGE, this);
	m_pageMisc.Create(IDD_CONFIG_MISC_PAGE, this);
	m_pageNetwork.Create(IDD_CONFIG_NETWORK_PAGE, this);
	m_pageLog.Create(IDD_CONFIG_LOG_PAGE, this);
	m_pageFolders.Create(IDD_CONFIG_FOLDERS, this);
    m_pageMailSmtp.Create(IDD_CONFIG_MAIL_SMTP, this);

	CRect rcDlgs;
	GetDlgItem(IDC_DLG_AREA)->GetWindowRect(rcDlgs);
	ScreenToClient(rcDlgs);
	
	rcDlgs.top		+= 12;
	rcDlgs.bottom	-= 5;
	rcDlgs.left		+= 5;
	rcDlgs.right	-= 5;
	
	m_pageAdvanced.MoveWindow(rcDlgs);  
	m_pageSecurity.MoveWindow(rcDlgs);  
	m_pageProxy.MoveWindow(rcDlgs);  
	m_pageEmailPref.MoveWindow(rcDlgs);
	m_pageEmail.MoveWindow(rcDlgs);
	m_pageMisc.MoveWindow(rcDlgs);
	m_pageNetwork.MoveWindow(rcDlgs);
	m_pageLog.MoveWindow(rcDlgs);
	m_pageFolders.MoveWindow(rcDlgs);
    m_pageMailSmtp.MoveWindow(rcDlgs);
	
	m_pageAdvanced.Init(data);
	m_pageSecurity.Init(data);
	m_pageProxy.Init(data);
	m_pageEmailPref.Init(data);
	m_pageEmail.Init(data);
	m_pageMisc.Init(data);
	m_pageNetwork.Init(data);
	m_pageLog.Init(data);
	m_pageFolders.Init(data);
    m_pageMailSmtp.Init(data);
	
	m_bDlgCreated = true;
	
	return TRUE;
}

void COptionsTreeDlg::OnSelchangingTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
	
	if(m_bDlgCreated)
	{
		HTREEITEM hCurrent = m_Tree.GetSelectedItem();
		
		if(hCurrent)
		{
			CCustomPropertyPage *page = ((CCustomPropertyPage*)m_Tree.GetItemData(hCurrent));
			
			if(page->ValidateData())
			{
				page->ShowWindow(SW_HIDE);
				page->SaveData(data);
			}
			else
			{
				*pResult = -1;
			}
		}
	}
}

void COptionsTreeDlg::OnSelchangedTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if(m_bDlgCreated)
	{
		HTREEITEM hCurrent = m_Tree.GetSelectedItem();
		
		if(hCurrent)
		{
			((CCustomPropertyPage*)m_Tree.GetItemData(hCurrent))->ShowWindow(SW_SHOW);
		}
	}
	
	*pResult = 0;
}

void COptionsTreeDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	delete pInfoImages;

	bConfigureVisible = FALSE;
}

void COptionsTreeDlg::OnOK() 
{
	HTREEITEM hCurrent = m_Tree.GetSelectedItem();
	
	if(hCurrent)
	{
		CCustomPropertyPage *page = ((CCustomPropertyPage*)m_Tree.GetItemData(hCurrent));
		
		if(page->ValidateData())
		{
			page->SaveData(data);
		}
		else
		{
			return;
		}
	}
	
	m_pageAdvanced.SaveData(data);
	m_pageSecurity.SaveData(data);
	m_pageProxy.SaveData(data);
	m_pageEmailPref.SaveData(data);
	m_pageEmail.SaveData(data);
	m_pageMisc.SaveData(data);
	m_pageNetwork.SaveData(data);
	m_pageLog.SaveData(data);
	m_pageFolders.SaveData(data);
    m_pageMailSmtp.SaveData(data);

	data.Save();
	
	bConfigureVisible = FALSE;

	CDialog::OnOK();
}

void COptionsTreeDlg::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	CMenu menu, *pSubMenu;

	if (menu.LoadMenu(IDR_POPUP_MENU) && (pSubMenu = menu.GetSubMenu(0))) {

		// Delete "configure" from menu, since we're already there
		pSubMenu->DeleteMenu(0, MF_BYPOSITION);
		// Delete separator as well
		pSubMenu->DeleteMenu(0, MF_BYPOSITION);

		//Display and track the popup menu
		pSubMenu->TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);

		menu.DestroyMenu();
	}

}

#endif // #ifdef WIN32

