/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ReadLocalizationFile.cpp
*
* Purpose : Class for reading and storing in memory internationalization
*           strings
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"
#include "ReadLocalizationFile.h"

extern LogFile logFile;

LocalizationInfo::LocalizationInfo():
    locDefault(NULL),
    bFreeLocDefault(false),
    locDataHead(NULL)
{
}

LocalizationInfo::~LocalizationInfo()
{
}

void LocalizationInfo::ReadFile()
{
	xmlDocPtr doc;
	xmlNodePtr cur;

#ifdef WIN32
    doc = xmlParseFile("localization.xml");
#else
	char filename[512];
	sprintf(filename,"%s/%s",CONFIG_FILE_LOC,"localization.xml");	
    doc = xmlParseFile(filename);
#endif

	if (doc == NULL)
    {
    }

	cur = xmlDocGetRootElement(doc);
	
	if (cur == NULL)
    {
		fprintf(stderr,"empty document\n");
		xmlFreeDoc(doc);
		return;
	}
	
	if (xmlStrcmp(cur->name, (const xmlChar *) "localization"))
    {
		fprintf(stderr,"document of the wrong type, root node != story");
		xmlFreeDoc(doc);
		return;
	}
	
	cur = cur->xmlChildrenNode;

	while (cur != NULL) 
    {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)"country")))
        {
            tempLocData.name.Format("%s", xmlGetProp(cur, (const xmlChar *)"name"));
            tempLocData.language.Format("%s", xmlGetProp(cur, (const xmlChar *)"language"));
			parseCountry(doc, cur);
		}

	    cur = cur->next;
	}

	xmlFreeDoc(doc);

    SetDefaultLocale();
}

void LocalizationInfo::parseCountry(xmlDocPtr doc, xmlNodePtr cur)
{
    xmlChar *key;
    cur = cur->xmlChildrenNode;

    while (cur != NULL) 
    {
        if ((!xmlStrcmp(cur->name, (const xmlChar *)"id")))
        {
		    key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		    tempLocData.id.Format("%s", key);
		    xmlFree(key);
	    }
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *)"regex")))
        {
		    key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		    tempLocData.regex.Format("%s", key);
		    xmlFree(key);
	    }
	    else if ((!xmlStrcmp(cur->name, (const xmlChar *)"format")))
        {
		    key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		    tempLocData.format.Format("%s", key);
		    xmlFree(key);
	    }
	
        cur = cur->next;
	}

	/* Add to List */
	LOCDATA *locDataTempPtr = new LOCDATA;
    locDataTempPtr->id       = tempLocData.id;
	locDataTempPtr->format   = tempLocData.format;
	locDataTempPtr->language = tempLocData.language;
	locDataTempPtr->name     = tempLocData.name;
	locDataTempPtr->regex    = tempLocData.regex;
	locDataTempPtr->next     = NULL;

	if(locDataHead == 0)
	{
		locDataHead = locDataTempPtr;
	}
	else
	{
		locDataTempPtr->next = locDataHead;
		locDataHead = locDataTempPtr;
	}

    tempLocData.id       = "";
	tempLocData.format   = "";
	tempLocData.language = "";
	tempLocData.name	 = "";
	tempLocData.regex    = "";
}

void LocalizationInfo::SetDefaultLocale()
{
    LocalizationInfo::LOCDATA* temp = GetFirst();

    while(temp != NULL)
	{
        if(temp->id == CStdString("us"))
        {
            locDefault = temp;
            break;
        }

        temp = temp->next;
	}

    if(locDefault == NULL)
    {
        logFile.Write(LOG_BASIC, "Setting default locale to hardcoded locale US");     
        locDefault = new LocalizationInfo::LOCDATA;
        locDefault->id       = "us";
		locDefault->format   = "Messages %d-%d of %d";
		locDefault->language = "English";
		locDefault->name     = "International (USA/Canada)";
		locDefault->regex    = "Messages [0-9]+-[0-9]+ of [0-9]+";
		locDefault->next     = NULL;
        bFreeLocDefault      = true;
    }
}

LocalizationInfo::LOCDATA* LocalizationInfo::GetDefaultLocale()
{
    return locDefault;
}

LocalizationInfo::LOCDATA* LocalizationInfo::GetFirst()
{
	return locDataHead;
}
