/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	WebBrowser.cpp
*
* Purpose : A simple class specifically designed to connect to
*           Yahoo! Mail and download/submit emails.
*
* Authors : Anuj Seth, Daniel Bryg, K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#include <openssl/md5.h>
#include "YAHOOPOPS.h"
#include "HttpClient.h"
#include "YahooStuff.h"
#include "DataHolder.h"
#include "relib_enhance.h"
#include "LogFile.h"
#include "YahooSessionCache.h"
#include "WebBrowser.h"

extern CDataHolder settings;
extern LogFile     logFile;
extern YahooSessionCache sessionCache;

#ifdef WIN32
CCriticalSection CWebBrowser::csConnect;
#endif

/*******************************************************************
* Function  : CWebBrowser::CWebBrowser
* Parameters: -
* Returns   : -
* Purpose   : Initialize the web browser object
*******************************************************************/
CWebBrowser::CWebBrowser():	
    msgHead(NULL),
    msgTail(NULL),
    msgCount(0),
	downloadedCount(0)
{
	//added by shyam on 15-06-04	
	// we shall add this here.the win32 versions use a global lock.
#ifndef WIN32
	pthread_mutex_init(&csConnect,NULL);
#endif	
	/* Dont create a new session everytime - Shane H.
	session = new HttpSession; */
}

/*******************************************************************
* Function  : CWebBrowser::~CWebBrowser()
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
CWebBrowser::~CWebBrowser()
{
	MESSAGE *ptr  = msgHead;
	MESSAGE *next = NULL;
	
	while(ptr != NULL)
	{
		next = ptr->next;
		delete ptr;
		ptr = next;
	}

	msgHead = NULL;
	msgTail = NULL;
}

/*******************************************************************
* Function  : ConnectToYahoo
* Parameters: user     -> Username sent by the POP3 client
*             password -> Password sent by the POP3 client
* Returns   : WEB_LOGIN_FAILED   -> Unable to login to Yahoo
*             WEB_UNKNOWN_ERROR  -> Possible bug in YahooPOPs! caused
*                                   by a redesign in the Yahoo site.
*		      WEB_WRONG_PASSWORD -> Incorrect password
*		      WEB_WRONG_USERNAME -> Incorrect login id
*             WEB_LOGIN_OK       -> Login successful
* Purpose   : To login to Yahoo's website.
*******************************************************************/
int CWebBrowser::ConnectToYahoo(char *user, char *pass)
{

	char data[1024];
	char *ptr = NULL;
	int	 retVal;

	username = user;
	password = pass;

	/* Edit SH : Avoid contention in connection between POP and SMTP threads
	Must be sure to unlock section before returning from function */
#ifdef WIN32
	csConnect.Lock();
#else
	pthread_mutex_lock(&csConnect);
#endif	

	/* EDIT: Shane .H : Attempt to obtain existing session
	Must wait until the other thread has finished connecting and saved the session 
	NOTE: This is not efficient if more than one username is connecting at once */
	YAHOO_SESSION *ySession = sessionCache.GetSession(username, password);
	if(ySession != NULL)
	{
		logFile.Write(LOG_ADVANCED, "<%s> Previous session recycled", LPCTSTR(username));
		SetSession(ySession);
#ifdef WIN32
		csConnect.Unlock();
#else
		pthread_mutex_unlock(&csConnect);
#endif
		return WEB_LOGIN_OK;
	} else
	{
		/* Create a new session if no existing session exists */
		logFile.Write(LOG_ADVANCED, "<%s> Creating a new session", LPCTSTR(username));
		session = new HttpSession;
	}

	/* Save cookies for future requests */
	session->SetSessionMgmt(TRUE);
   
	HttpClient c(session);
	HttpClientResponse resp;
	
	logFile.Write(LOG_BASIC, "<%s> Connecting to Yahoo...", LPCTSTR(username));
	
	if(settings.m_bUseProxy)
	{
		logFile.Write(LOG_ADVANCED, "<%s> Using proxy settings [%s:%d / %s:%d]", 
			LPCTSTR(username), 
			LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort,
			LPCTSTR(settings.m_strHttpsProxyHost), settings.m_nHttpsProxyPort);
		
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
#ifdef WIN32		
		if(!settings.m_strHttpsProxyHost.TrimRight().IsEmpty())
#else
		if(settings.m_strHttpsProxyHost[0] != '\0')
#endif
		{
			c.SetHttpsProxy(LPCTSTR(settings.m_strHttpsProxyHost), settings.m_nHttpsProxyPort);
			c.UseHttpsProxy(TRUE);
		}
	}
	
	if(settings.m_bUseMD5)
	{
		logFile.Write(LOG_ADVANCED, "<%s> Using MD5 secured password transmission", LPCTSTR(username));
		
		char challengeText[128];
		/* 
		* Secure Password Transmission is enabled.
		* This is a slower operation but more secure.
		* The reason being that we first need to fetch
		* the http://mail.yahoo.com/ page and then 
		* login. We need to extract the ".challenge"
		* text from the page. After that we do the 
		* actual login. In the other process (non-MD5)
		* we do a login directly as there is no 
		* challenge text involved.
		*/
		if(c.DoGet("http://mail.yahoo.com", resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
#ifdef WIN32
			csConnect.Unlock();
#else
			pthread_mutex_unlock(&csConnect);
#endif
			return WEB_LOGIN_FAILED;
		}
		
		ptr    = NULL;
		retVal = SearchRegExp(resp.GetBodyPtr(), 
			"name=\".challenge\" value=\"", 
			&ptr,
			"name=\".challenge\" value=\"%s\"", 
			challengeText);
		
		if(retVal != 0)
		{
			/* Something's wrong! */
			logFile.Write(LOG_BASIC, "<%s> Login failed (Something's wrong!)", LPCTSTR(username));
#ifdef WIN32
			csConnect.Unlock();
#else
			pthread_mutex_unlock(&csConnect);
#endif
			return WEB_LOGIN_FAILED;
		}
		
		ptr = strchr(challengeText, '\"');
		
		if(ptr != NULL)
		{
			*ptr = '\0';
		}
		
		/*
		 * The following implementation of the hashing technique is 
		 * as per Yahoo's implementation which is available in the
		 * Javascript code at http://mail.yahoo.com/
		 */
		
		char hash[256];
		
		MD5_CTX ctx;
		MD5_Init(&ctx);
		MD5_Update(&ctx, password, strlen(password));
		MD5_Final(reinterpret_cast<unsigned char *>(hash), &ctx);
		
		hash[0] = '\0';
		rhex(ctx.A, data);
		strcat(hash, data);
		rhex(ctx.B, data);
		strcat(hash, data);
		rhex(ctx.C, data);
		strcat(hash, data);
		rhex(ctx.D, data);
		strcat(hash, data);
		strcat(hash, challengeText);
		
		MD5_Init(&ctx);
		MD5_Update(&ctx, hash, strlen(hash));
		MD5_Final(reinterpret_cast<unsigned char *>(hash), &ctx);
		
		hash[0] = '\0';
		rhex(ctx.A, data);
		strcat(hash, data);
		rhex(ctx.B, data);
		strcat(hash, data);
		rhex(ctx.C, data);
		strcat(hash, data);
		rhex(ctx.D, data);
		strcat(hash, data);
		
		c.Reset();
		resp.Reset();
		
		if(settings.m_bSecureMode)
		{
			sprintf(data, "%s?.tries=1&.src=ym&.intl=us&login=%s&passwd=%s&.hash=1&.md5=1&.js=1&.challenge=%s&.persistent=y", yahoo[HTTPS_LOGIN], LPCTSTR(UrlEncode(user)), hash, challengeText);
			
			if(c.DoGet(data, resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> MD5 based login failed in secure mode", LPCTSTR(username));
#ifdef WIN32
				csConnect.Unlock();
#else
				pthread_mutex_unlock(&csConnect);
#endif
				return WEB_LOGIN_FAILED;
			}
		}
		else
		{
			sprintf(data, "%s?.tries=1&.src=ym&.intl=us&login=%s&passwd=%s&.hash=1&.md5=1&.js=1&.challenge=%s&.persistent=y", yahoo[HTTP_LOGIN], LPCTSTR(UrlEncode(user)), hash, challengeText);
			
			if(c.DoGet(data, resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> MD5 based login failed", LPCTSTR(username));
#ifdef WIN32	
				csConnect.Unlock();
#else
				pthread_mutex_unlock(&csConnect);
#endif
				return WEB_LOGIN_FAILED;
			}
		}
	}
	else
	{
		/* Normal connect without MD5 password hash */
		sprintf(data, ".tries=1&.src=ym&.intl=us&login=%s&passwd=%s&.persistent=y", LPCTSTR(UrlEncode(user)), LPCTSTR(UrlEncode(password)));
		
		if(settings.m_bSecureMode)
		{
			if(c.DoPost(yahoo[HTTPS_LOGIN], data, resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> Login failed in secure mode", LPCTSTR(username));
#ifdef WIN32	
				csConnect.Unlock();
#else
				pthread_mutex_unlock(&csConnect);
#endif
				return WEB_LOGIN_FAILED;
			}
		}
		else
		{
			if(c.DoPost(yahoo[HTTP_LOGIN], data, resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
#ifdef WIN32	
				csConnect.Unlock();
#else
				pthread_mutex_unlock(&csConnect);
#endif
				return WEB_LOGIN_FAILED;
			}
		}
	}
	
	c.GetHost(host);
	
	/* Check if the login was successful */
	if(resp.DataContains("Invalid Password"))
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed due to invalid password", LPCTSTR(username));
#ifdef WIN32	
		csConnect.Unlock();
#else
		pthread_mutex_unlock(&csConnect);
#endif
		return WEB_WRONG_PASSWORD;
	}
	
	if(resp.DataContains("ID does not exist"))
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed due to invalid user id", LPCTSTR(username));
#ifdef WIN32	
		csConnect.Unlock();
#else
		pthread_mutex_unlock(&csConnect);
#endif
		return WEB_WRONG_USERNAME;
	}
	
	/* redirect to the first link if still on login page 
	 - this happens in secure mode */
	if(host == "login.yahoo.com")
	{
		/* Extract URL from return value */
		char *endPtr = '\0';
		char *startPtr = resp.GetBodyPtr();
		char url[1024];
		char tempHost[1024];
		url[0] = '\0';

		/* addition by K.Shyam for a nasty bug !.
		 * we assumed that the first url in this page is the login url
		 * but it is not so.so search till we get the login url
		 */
		while ( 1 )
		{
			if ( SearchRegExp(startPtr, "http://",&endPtr, "%s", url) != 0 )
				break;/*we didnt have the regex inside */
			startPtr = endPtr + 7;/* move out of http:// */
			endPtr = '\0';
			char *ptr2 = strchr(url, '\"');
			
			if(ptr2 != NULL)
			{
#ifdef WIN32
				*ptr2 = NULL;
#else
				*ptr2 = '\0';
#endif
			}
			c.SplitUrl(url,tempHost);
			if ( strstr(tempHost,"mail.yahoo.com") != NULL )
				break;
		}
		
		resp.Reset();
		c.Reset();
		if(c.DoGet(url, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
#ifdef WIN32	
			csConnect.Unlock();
#else
			pthread_mutex_unlock(&csConnect);
#endif
			return WEB_LOGIN_FAILED;
		}
		
		c.GetHost(host);
	}

	logFile.Write(LOG_ADVANCED, "<%s> Connected to %s.", LPCTSTR(username), LPCTSTR(host));

	/* host value should be in mail.yahoo.com domain */
	if(host.Find(".mail.yahoo.com")<0)
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed. No redirection to *.mail.yahoo.com.", LPCTSTR(username));

		DumpPage("login",resp.GetBodyPtr());

#ifdef WIN32	
		csConnect.Unlock();
#else
		pthread_mutex_unlock(&csConnect);
#endif
		return WEB_UNKNOWN_ERROR;
	}

	/* Extract the crumb value */
	ptr = NULL;
	char  crumb[64];
	retVal = SearchRegExp(resp.GetBodyPtr(), 
		"ET=1&.crumb=", &ptr,
		"ET=1&.crumb=%s&", crumb);

	if(retVal == 0)
	{
		ptr = strchr(crumb, '&');
		
		if(ptr != NULL)
		{
			*ptr = '\0';
		}
		crumbValue = crumb;
				
		logFile.Write(LOG_ADVANCED, "<%s> Crumb extracted from empty-trash link", LPCTSTR(username));
	} else
		logFile.Write(LOG_BASIC, "<%s> Could not extract crumb from empty-trash link", LPCTSTR(username));

	sessionCache.AddSession(username, password, crumbValue, session, host);
#ifdef WIN32
	csConnect.Unlock();
	/* Other thread may now attempt to access section */
#else
	pthread_mutex_unlock(&csConnect);
#endif

	return WEB_LOGIN_OK;
}

/*******************************************************************
* Function  : FetchInboxList()
* Parameters: mailbox - The name of the mailbox (INBOX, BulkMail)
* Returns   : WEB_INBOX_FETCH_FAILED - Unable to retrieve email list
*             WEB_INBOX_EMPTY        - No emails in Inbox :(
*             WEB_INBOX_FETCH_OK     - Success :)
* Purpose   : To determine the number of emails in the Inbox and
*             to make the URL list for each email so that it can
*             be downloaded.
*******************************************************************/
int CWebBrowser::FetchMailboxList(const CStdString &folderName)
{
	int	  page		     = 0;
	char *ptr			 = NULL;
	int	  retVal		 = 0;
	char  inboxUrl[1024];
	unsigned int emailCtr = 0;
	
	HttpClient c(session);
	HttpClientResponse resp;
	
	if(settings.m_bUseProxy)
	{
		c.SetHttpProxy(settings.m_strHttpProxyHost, settings.m_nHttpProxyPort);
		c.UseHttpProxy(TRUE);
	}
	
	/* Loop through the pages of the folder */
	while (settings.m_bIgnoreSize || (msgCount < settings.m_nMaxEmails))
	{
		sprintf(inboxUrl, yahoo[MBOX], LPCTSTR(host), LPCTSTR(UrlEncode(folderName)), page, settings.m_nEmailCategory);
		
		if(c.DoGet(inboxUrl, resp) != 0)
		{
			return WEB_INBOX_FETCH_FAILED;
		}

		/* A section dealing with locale parsing
		   has been removed (by ltD) */

		/* Extract all the message Id's from the page */
		char msgId[192];
		MESSAGE* msgPtr  = NULL;
		MESSAGE* msgTemp = NULL; 
		char *ptr2		 = '\0';
		BOOL bFound		 = FALSE;

		/* Search for mailbox usage */
		ptr = NULL;
		char boxUsage[10];
		retVal = SearchRegExp(resp.GetBodyPtr(), 
		        "<td bgcolor=\"green\" width=\"[0-9]+%\"", &ptr, "<td bgcolor=\"green\" width=\"%s", boxUsage);
		if ((ptr = strchr(boxUsage, '%'))!=NULL) {
			*ptr=0;
			logFile.Write(LOG_BASIC, "<%s> You use %s%%%% of your Yahoo! mailbox", LPCTSTR(username), boxUsage);
		}
		
		ptr = NULL;

		/* Get id's from the current page */
		do
		{

			retVal = SearchRegExp(resp.GetBodyPtr(), 
				        "MsgId=[a-zA-Z0-9_]*", &ptr, "MsgId=%s&YY", msgId);
			
			ptr++;
			
			if(retVal == 0)
			{
				ptr2 = strchr(msgId, '&');
			
				if(ptr2 != NULL)
				{
					*ptr2		 = '\0';
				}
			
    			/* 
	    	 	 * Check if this message already exists
		     	 * in the list. If not, then add it.
				 */
				msgTemp = msgHead;
				bFound  = FALSE;
				
				while(msgTemp != NULL)
				{
					if(msgTemp->msgId == msgId)
					{
						bFound = TRUE;
						break;
					}
					
					msgTemp = msgTemp->next;
				}
				
				if(bFound)
				{
					continue;
				}
				
				msgPtr = new MESSAGE;
				msgPtr->next = NULL;
				msgPtr->msgId = msgId;
				msgPtr->mailbox = folderName;
				msgPtr->email.SetTempSize(1024);

				/* message id is a sequence of numbers
				   separated by underscore - second and third
				   of those may change so remove them
				   for UIDL purposes
				   (experimental change by ltd) */
				int hyppos=msgPtr->msgId.Find('_');
				CStdString msgshortid=msgPtr->msgId.Left(hyppos);
				hyppos=msgPtr->msgId.Find('_',hyppos+1);
				msgshortid+=msgPtr->msgId.Mid(msgPtr->msgId.Find('_',hyppos+1));
			    msgPtr->email.SetMsgId(msgshortid);
				
				if(msgHead == NULL)
				{
					msgHead = msgPtr;
					msgTail = msgPtr;
				}
				else
				{
					msgTail->next = msgPtr;
					msgTail = msgPtr;
				}
				
				msgCount++;
                emailCtr++;
			}	
			
			if(!settings.m_bIgnoreSize && (msgCount >= settings.m_nMaxEmails))
			{
			/* 
			* We cannot download any more emails for
			* this session as we have hit the max
			* limit.
				*/
				logFile.Write(LOG_ADVANCED, "<%s> Message limit (%d) for session reached", LPCTSTR(username), settings.m_nMaxEmails);
				break;
			}
		} while(retVal == 0);
		
		/* Dump the folder page for debugging
		   if no id's were extracted */
		if (!emailCtr) {
			logFile.Write(LOG_BASIC, "<%s> No messages were found. Folder probably empty", LPCTSTR(username));
			DumpPage("inbox", resp.GetBodyPtr());
		}

		/* Scan the page for a part of a link to the next one */
		ptr = strstr(resp.GetBodyPtr(), "&next=1&");
		
		/* Don't download the next page if limit has been hit
		   or there is no next page */
		if((ptr==NULL) || (!settings.m_bIgnoreSize && (msgCount >= settings.m_nMaxEmails)))
			break;

		page++;
		
		c.Reset();
		resp.Reset();
	}
	
	logFile.Write(LOG_ADVANCED, "<%s> Extracted %d message Id's from mailbox %s", LPCTSTR(username), emailCtr, LPCTSTR(folderName));
	
	return WEB_INBOX_FETCH_OK;
}

/*******************************************************************
* Function  : GetMsgCount()
* Parameters: - 
* Returns   : The number of emails in the mailbox
* Purpose   : To return the count of the number of emails in the
*             users mailbox.
*******************************************************************/
int CWebBrowser::GetMsgCount()
{
	return msgCount;
}

/*******************************************************************
* Function  : DownloadAllEmails()
* Parameters: progress - Reference to the progress bar dialog
* Returns   : 0
* Purpose   : To download each and every email in the Yahoo 
*             mailbox
*******************************************************************/
#ifdef WIN32
int CWebBrowser::DownloadAllEmails(CProgressWnd *progress)
#else
int CWebBrowser::DownloadAllEmails()
#endif
{

#ifdef WIN32	
	CStdString		   progressStr;
#endif
	MESSAGE			   *msg = msgHead;
	int				   ctr = 1;
	
	while(msg != NULL)
	{
#ifdef WIN32
		if(progress != NULL)
		{
			progressStr.Format("Downloading email %d of %d", ctr, GetMsgCount());
			progress->SetText(progressStr);
			progress->PeekAndPump();
		}
#endif
		logFile.Write(LOG_BASIC, "<%s> Downloading email %d", LPCTSTR(username), ctr);
		
		ctr++;
		
		DownloadEmail(msg);

		msg = msg->next;

#ifdef WIN32
		if(progress != NULL)
		{
			progress->StepIt();
			
			if(progress->Cancelled())
			{
				/* User clicked on the Cancel button */
				return 1;
			}
		}
#endif
	}
	
	return 0;
}

/*******************************************************************
* Function  : DownloadEmail()
* Parameters: msg - message to be downloaded
*             heasersonly - default FALSE
* Returns   : 0
* Purpose   : To download one message
*******************************************************************/
int CWebBrowser::DownloadEmail(MESSAGE *msg, BOOL headersonly)
{
	char			   *ptr = NULL;
	char			   url[1024];
	HttpClient		   c(session);
	HttpClientResponse resp;
	
	/* return error if the limit of downloaded emails has been reached */
	if (settings.m_bIgnoreSize && (downloadedCount>=settings.m_nMaxEmails)) 
		return 1;

	if(settings.m_bUseProxy)
	{
		c.SetHttpProxy(settings.m_strHttpProxyHost, settings.m_nHttpProxyPort);
		c.UseHttpProxy(TRUE);
	}
	
	/* Download the email header */
	if (!msg->email.IsHeaderDownloaded()) {
		sprintf(url, yahoo[MSG], LPCTSTR(host), LPCTSTR(UrlEncode(msg->mailbox)), LPCTSTR(msg->msgId), "HEADER");
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), url);
	
		/* Edit SH. Make sure content-type contains 'filename' to check that 
		the request was processed correctly */
		if((c.DoGet(url, resp) == 0) && resp.ContentTypeContains("filename"))
		{
		    msg->email.SetHeader(resp.GetBodyPtr());

			/* Add X-YahooPOPs-Folder header */
			msg->email.AddHeader("X-YahooPOPs-Folder", LPCTSTR(msg->mailbox));
		}
		else
		{
			/* The headers couldn't be downloaded so skip this email */
			logFile.Write(LOG_ADVANCED, "<%s> Headers could not be downloaded; skipping message", LPCTSTR(username));
			DumpPage("messageheaders", resp.GetBodyPtr());
			return 1;
		}
	}
	
	/* The headers have been downloaded, so increase the count */
	downloadedCount++;

	c.Reset();
	resp.Reset();

	/* Download the email body (including attachments) */
	if (!headersonly) {
		sprintf(url, yahoo[MSG], LPCTSTR(host), LPCTSTR(UrlEncode(msg->mailbox)), LPCTSTR(msg->msgId), "TEXT");
	
		/* SH. Used for testing invalid msgs/bandwidth over quota etc
		sprintf(url, yahoo[MSG], LPCTSTR(host), LPCTSTR(UrlEncode(msg->mailbox)), "BUNCHO_CRAP", "TEXT");
		*/
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), url);
		
		if((c.DoGet(url, resp) == 0) && resp.ContentTypeContains("filename"))
		{
			/* if body was encoded by sender (Yahoo has decoded it), encode it again */
			if (msg->email.GetHeader("Content-Transfer-Encoding").ToLower()=="base64") {
				DwString dwbody;
				dwbody.assign(resp.GetBodyPtr(),resp.GetBodyLength());
				DwEncodeBase64(dwbody,dwbody);
				msg->email.SetBody(dwbody.c_str());
			} else {
				CStdString data=resp.GetBodyPtr();
				/* Replace LF with CRLF unless the email is in text format */
				if (msg->email.GetHeader("Content-Type").Left(10).ToLower()!="text/plain")
					data.Replace("\n","\r\n");

				/* Edit SH. Replace .'s with ..'s for POP3 conformance */
				data.Replace("\r\n.", "\r\n..");
				if (data[0]=='.') data.Insert(0,'.');

				msg->email.SetBody(data.GetBuffer());
			}

		}
		else
		{
			/* The email body could not be downloaded, so skip this message */
			logFile.Write(LOG_ADVANCED, "<%s> Email could not be downloaded", LPCTSTR(username));
			DumpPage("messagebody", resp.GetBodyPtr());
			return 1;
		}
	}

	return 0;
}

/*******************************************************************
* Function  : DownloadEmail()
* Parameters: msgNumber - message to be downloaded
*             heasersonly - default FALSE
* Returns   : 0
* Purpose   : Overloaded version finds the right pointer
			  and downloads one message
*******************************************************************/
int CWebBrowser::DownloadEmail(int msgNumber, BOOL headersonly)
{
	MESSAGE *ptr=msgHead;

	if(msgNumber > GetMsgCount())
		return 1;

	for (int i=1;i<msgNumber;i++) ptr=ptr->next;

	return DownloadEmail(ptr, headersonly);
}

/*******************************************************************
* Function  : GetEmail
* Parameters: msgNumber - The email to retrieve
* Returns   : Pointer to an Email object or NULL if it could not
*             be found
* Purpose   : To retrieve a specific email from the list
*******************************************************************/
Email* CWebBrowser::GetEmail(int msgNumber)
{
	MESSAGE *ptr  = msgHead;
	MESSAGE *prev = msgHead;
	
	if(msgNumber > GetMsgCount())
		/* Invalid */
		return NULL;
	
	for(int i = 0; i < msgNumber && ptr != NULL; i++)
	{
		prev = ptr;
		ptr  = ptr->next;
	}
	
	if (prev!=NULL) return &(prev->email);
	else return NULL;
}

/*******************************************************************
* Function  : AnyEmailsToDelete()
* Parameters: -
* Returns   : true - there are emails to delete; false otherwise
* Purpose   : To determine if there any any emails to delete
*******************************************************************/
bool CWebBrowser::AnyEmailsToDelete()
{
    if(settings.m_bLeaveUnread)
    {
        return false;
    }

	MESSAGE	*ptr = msgHead;

   	while(ptr != NULL)
	{
		if(ptr->email.IsDeleted())
		{
            return true;
		}
		
		ptr = ptr->next;
    }

    return false;
}

/*******************************************************************
* Function  : DeleteEmails()
* Parameters: -
* Returns   : -
* Purpose   : To delete all messages that have been marked as 
*             deleted by the DELE command during the session.
*******************************************************************/
void CWebBrowser::DeleteEmails(const CStdString &folder)
{
	/* Delete all the emails now. */
	MESSAGE				*ptr = msgHead;
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			delUrl;
	
	if(settings.m_bLeaveUnread)
	{
		logFile.Write(LOG_ADVANCED, "<%s> Deletion canceled because of leave messages on server option", LPCTSTR(username));
		return;
	}
   
    if(settings.m_bUseProxy)
	{
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
	}
	
	int ctr = 0;
	
	delUrl.Format(yahoo[DEL], LPCTSTR(host), LPCTSTR(UrlEncode(folder)));
	
	while(ptr != NULL)
	{
		if(ptr->email.IsDeleted() && ptr->mailbox == folder)
		{
			delUrl += "&Mid=";
			delUrl += ptr->msgId;
			ctr++;
		}
		
		ptr = ptr->next;
		
		if((ctr==5) || (ctr && (ptr==NULL)))
		{		
			/* Add the crumb */
			delUrl += "&.crumb=";
			delUrl += crumbValue;
			
			c.DoGet(delUrl, resp);
			c.Reset();
			resp.Reset();
			
			logFile.Write(LOG_ADVANCED, "<%s> Deleting emails from Yahoo", LPCTSTR(username));
			logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(delUrl));
			
			ctr = 0;
			delUrl.Format(yahoo[DEL], LPCTSTR(host), LPCTSTR(UrlEncode(folder)));
		}
	}
}

/*******************************************************************
* Function  : rhex
* Parameters: num - A number from the MD5 context (A, B, C or D)
* Returns   : A hex string with ls-byte first
* Purpose   : Convert a 32-bit number to a hex string with ls-byte 
*             first. 
*******************************************************************/
inline void CWebBrowser::rhex(unsigned int num, char *data)
{
	CStdString hex_chr = "0123456789abcdef";
	CStdString str;
	
	for(int j = 0; j <= 3; j++)
	{
		str += hex_chr[(num >> (j * 8 + 4)) & 0x0F];
		str += hex_chr[(num >> (j * 8)) & 0x0F];
	}
	
	strcpy(data, str);
}

/*******************************************************************
* Function  : EmptyTrash
* Parameters: -
* Returns   : -
* Purpose   : Empty the Yahoo Trash folder
*******************************************************************/
void CWebBrowser::EmptyTrash()
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			trashUrl;
	
    if(settings.m_bUseProxy)
	{
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
	}

    if(GetMsgCount() > 0)
	{
		/* Empty trash only if some emails were downloaded */
		trashUrl.Format(yahoo[TRASH], LPCTSTR(host), LPCTSTR(crumbValue));
		
		logFile.Write(LOG_BASIC, "<%s> Emptying trash", LPCTSTR(username));
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(trashUrl));
		
		c.DoGet(trashUrl, resp);
	}
}

void CWebBrowser::EmptyBulkMail()
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			trashUrl;
	
    if(settings.m_bUseProxy)
	{
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
	}

    trashUrl.Format(yahoo[EMPTY_BULKMAIL], LPCTSTR(host), LPCTSTR(crumbValue));
	
	logFile.Write(LOG_BASIC, "<%s> Emptying bulk mail folder", LPCTSTR(username));
	logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(trashUrl));
	
	c.DoGet(trashUrl, resp);
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::MarkUnread(const CStdString &mailbox)
{
	/* Delete all the emails now. */
	MESSAGE				*ptr = msgHead;
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			url;
	bool				bUnread = false;
	
	if(settings.m_bUseProxy)
	{
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
	}
	
	url.Format(yahoo[UNREAD], LPCTSTR(host), LPCTSTR(mailbox), LPCTSTR(crumbValue));
	
	while(ptr != NULL)
	{
		if(!ptr->email.IsDeleted() && ptr->mailbox == mailbox)
		{
			url += "&Mid=";
			url += ptr->msgId;
			bUnread = true;
		}
		
		ptr = ptr->next;
	}
	
	if(bUnread)
	{
		c.DoGet(url, resp);
		logFile.Write(LOG_ADVANCED, "<%s> Marking messages as unread (Mailbox = %d)", LPCTSTR(username), LPCTSTR(mailbox));
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(url));
	}
	else
	{
		logFile.Write(LOG_ADVANCED, "<%s> There are no emails to mark as unread", LPCTSTR(username));
	}
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::SetSession(YAHOO_SESSION *ySession)
{
	this->session = ySession->session;
	this->host = ySession->host;
	this->username = ySession->username;
	this->password = ySession->password;
	this->crumbValue = ySession->crumbValue;
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::SaveSession()
{
	sessionCache.AddSession(username, password, crumbValue, session, host);
}

/*******************************************************************
* Function  : SendMail
* Parameters: to, cc, bcc - email recipients, subject - email title
			  body - email contents, htmlbody - set if rich text format
			  EmailAttachmentList - mimepp list of attachments
* Returns   : 0 - success, 1 - retrieving of compose page failed
			  2 - email submission failed, 3 - attachment submission failed
* Purpose   : To submit an email message to Yahoo! for sending
*******************************************************************/
int CWebBrowser::SendEmail(CStdString &to, CStdString &cc, 
						   CStdString &bcc, CStdString &subject, 
						   CStdString &body, bool htmlbody,
						   const EmailAttachmentList &attachment)
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			url;
	CStdString			postData;
	CStdString			contentLength;
	char				*ptr = NULL;
	char				crumb[128];
	char				NumAtt[16]="0";
	char				AttData[1024]="";
	int					retVal;
	char				yy[128];
	
	if(settings.m_bUseProxy)
	{
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
	}
    
	/* Get the Compose page */
	url.Format(yahoo[COMPOSE_GET], LPCTSTR(host));

	if(c.DoGet(url, resp) != 0) {
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not download compose page", LPCTSTR(username), LPCTSTR(to));
		return 1;
	}
	
	/* Extract crumb */
	retVal = SearchRegExp(resp.GetBodyPtr(), 
		"name=\".crumb\"", &ptr,
		"name=\".crumb\" value=\"%s\"", crumb);

	if ((retVal == 0) && (ptr = strchr(crumb, '\"')) != NULL)
	{
		*ptr = '\0';
	}
	
	/* Extract YY */
	ptr = NULL;
	/* If crumb or YY extraction has failed dump the compose page to a file */
	if (0!=SearchRegExp(resp.GetBodyPtr(), "YY=", &ptr, "YY=%s", yy)
		|| retVal!=0) {
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not analyze compose page", LPCTSTR(username), LPCTSTR(to));
		/* Dump the compose page */
		DumpPage("compose", resp.GetBodyPtr());
		return 1;
	}

	if((ptr = strchr(yy, '\"')) != NULL) *ptr = '\0';

	/* Send attachments (if any) */
	if(attachment.Count() > 0)
	{
		/* prepare multipart form */
		struct curl_httppost *formpost=NULL;
		struct curl_httppost *lastptr=NULL;
		CStdString Formfiletype;
		CStdString Formfilename;
		int reqnumatt=0;	//number of posted attachments

		/* iterate all requested attachments with retVal as counter */
		for(retVal = 0; retVal < attachment.Count() ; retVal++) {

			/* if there's no filename it's not a real attachment.
			   For instance, mimepp treats html parts of rich text emails
			   as attachments. */
			if (!attachment.Get(retVal).FileName().empty()) {
				Formfiletype.Format("%s/%s",
					attachment.Get(retVal).Type().c_str(),attachment.Get(retVal).Subtype().c_str());

				/* Ugly trick here to include file name header 
				   in form field name header. */
				Formfilename.Format("userFile%d\"; filename=\"%s", retVal, attachment.Get(retVal).FileName().c_str());

				/* add attchment to multipart form */
				curl_formadd(&formpost, &lastptr,
					CURLFORM_COPYNAME, Formfilename.c_str(),
					CURLFORM_CONTENTTYPE, Formfiletype.c_str(),
					CURLFORM_PTRCONTENTS, attachment.Get(retVal).Content().c_str(), 
					CURLFORM_CONTENTSLENGTH, attachment.Get(retVal).Size(),
					CURLFORM_END);

				reqnumatt++;
			} //if
		} //for
		
		/* Proceed with attachments only when good ones have been found */
		if (reqnumatt) {
			/* add upload request to multipart form */
			curl_formadd(&formpost, &lastptr,
				CURLFORM_COPYNAME, "UPL",
				CURLFORM_COPYCONTENTS, "1", 
				CURLFORM_END);

			/* send the form */
			c.Reset();
			resp.Reset();
		
			url.Format(yahoo[COMPOSE_ATTACH], LPCTSTR(host), yy);

			/* the last parameter indicates posting multipart/form-data */
			if (c.DoPost(url, formpost, resp, true)!=0) {
				
				curl_formfree(formpost);
				
				logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not upload attachments", LPCTSTR(username), LPCTSTR(to));
				return 3;
			}
			
			curl_formfree(formpost);
			
			/* Extract YY */
			ptr = NULL;
			retVal = SearchRegExp(resp.GetBodyPtr(), "YY=", &ptr, "YY=%s", yy);
			if(retVal == 0 && (ptr = strchr(yy, '\"')) != NULL)
			{
				*ptr = '\0';
			}
			
			/* Extract NumAtt */
			ptr = NULL;
			retVal = SearchRegExp(resp.GetBodyPtr(), "name=NumAtt value=\"", &ptr, "name=NumAtt value=\"%s", NumAtt);
			if(retVal == 0 && (ptr = strchr(NumAtt, '\"')) != NULL)
			{
				*ptr = '\0';
			}
			
			/* Extract AttData */
			if (NULL != (ptr = strstr(resp.GetBodyPtr(), "name=AttData value=\""))) {
				/* Skip to the value of "value" */
				ptr +=sizeof("name=AttData value=\"") - 1;
				/* cut off the quote */
				retVal=strcspn(ptr, "\"");
				strncpy(AttData, ptr, retVal);
				AttData[retVal]=0;
				/* Encode */
				strcpy(AttData, UrlEncode(AttData));
			}

			/* check the number of successfully uploaded attachments */
			if (retVal=(reqnumatt-atoi(NumAtt))) {
				
				logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. %d attchments not uploaded", LPCTSTR(username), LPCTSTR(to), retVal);

				/* Dump the attachment page */
				DumpPage("attachments", resp.GetBodyPtr());
					
				/* Send a cancel request */
				url.Format(yahoo[COMPOSE_POST], LPCTSTR(host), yy);
				
				c.Reset();
				resp.Reset();
				
				postData.Format(".crumb=%s&NumAtt=%s&AttData=%s&CAN=Cancel", 
					crumb, NumAtt, AttData);
				
				contentLength.Format("%d", postData.GetLength());
				c.AddHeader("Content-Length", LPCTSTR(contentLength));

				/* Log a successful cancel request */
				if (c.DoPost(url, LPCTSTR(postData), resp)==0)
					logFile.Write(LOG_ADVANCED, "<%s> Attachments cancelled", LPCTSTR(username));
				
				return 3;
			} //if (i=(reqnumatt-atoi(NumAtt)))
		} //if (reqnumatt)
	} //if(attachment.Count() > 0)

	/* Edit SH - Change ..'s to .'s. */
	if (body[0]=='.') body.Delete(0);
	body.Replace("\r\n..", "\r\n.");
	
	/* Generate response page with email information */

	/* Yahoo does not accept empty To recipients so set it to "" */
	if (to.IsEmpty()) to="\"\"";
	char * escTo  = curl_escape(LPCTSTR(to), 0);
	char * escCc  = curl_escape(LPCTSTR(cc), 0);
	char * escBcc = curl_escape(LPCTSTR(bcc), 0);
	char * escSub = curl_escape(LPCTSTR(subject), 0);
	char * escBody= curl_escape(LPCTSTR(body), 0);

	postData.Format(".crumb=%s&To=%s&Cc=%s&Bcc=%s&Subj=%s&Body=", 
                    crumb, escTo, escCc, escBcc, escSub);
    postData += escBody;
	postData += "&NumAtt=";
	postData += NumAtt;
	postData += "&AttData=";
	postData += AttData;

	postData += "&box=&FwdFile=&FwdMsg=&FwdSubj=&OriginalFrom=&OriginalSubject=&InReplyTo=&FName=&Video=&ATT=&VID=&Format=";
	postData += htmlbody ? "html" : "";
	postData += "&SigAtt=&SaveCopy=";
	postData += settings.m_bSaveEmailCopy ? "1" : "";
	postData += "&NoSubj=1&NoBody=1&SEND=%A0%A0Send%A0%A0\n";

	/* free data */
	curl_free(escTo);
	curl_free(escCc);
	curl_free(escBcc);
	curl_free(escSub);
	curl_free(escBody);

	url.Format(yahoo[COMPOSE_POST], LPCTSTR(host), yy);

	c.Reset();
	resp.Reset();
	
	contentLength.Format("%d", postData.GetLength());
	c.AddHeader("Content-Length", LPCTSTR(contentLength));
	c.AddHeader("Content-Type", "application/x-www-form-urlencoded");

	/* send the message */
	if(c.DoPost(url, LPCTSTR(postData), resp) != 0)
	{
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not submit message", LPCTSTR(username));
		return 2;
	}
	
	/* Retrieved page should contain AddAddresses form even if
	   the address is already in the book */
	if (strstr(resp.GetBodyPtr(), "name=\"AddAddresses\"")==NULL) 
	{
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Yahoo! has not accepted the message", LPCTSTR(username), LPCTSTR(to));

		/* Dump the response page to a file so that it is easy to analyze it */
		DumpPage("sendresponse", resp.GetBodyPtr());

		/* Does the page contain YY ? */
		ptr = NULL;
		if (SearchRegExp(resp.GetBodyPtr(), "YY=", &ptr, "YY=%s", yy)==0) {

			if ((ptr = strchr(yy, '\"')) != NULL)
			{
				*ptr = '\0';
			}

			/* Send a cancel request */
			url.Format(yahoo[COMPOSE_POST], LPCTSTR(host), yy);
		
			c.Reset();
			resp.Reset();
		
			postData.Format(".crumb=%s&NumAtt=%s&AttData=%s&CAN=Cancel", 
			crumb, NumAtt, AttData);
		
			contentLength.Format("%d", postData.GetLength());
			c.AddHeader("Content-Length", LPCTSTR(contentLength));
		
			/* Log a successful cancel request */
			if (c.DoPost(url, LPCTSTR(postData), resp)==0)
				logFile.Write(LOG_ADVANCED, "<%s> Message cancelled", LPCTSTR(username));
			else logFile.Write(LOG_ADVANCED, "<%s> Message not cancelled", LPCTSTR(username));
		}

		return 2;
	}

	return 0;
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
CStdString CWebBrowser::UrlEncode(const char *data)
{
	char *escapedUrl = curl_escape(data, 0);
	CStdString url(escapedUrl);
	curl_free(escapedUrl);

	return url;
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::DeleteSession()
{
	//Remove it from the cache.
	sessionCache.DeleteSession(username);
    //Above line should delete this session, but it may not exist in the session
	//cache yet, so double check.
	if (session != NULL)
		delete session;
    session = NULL;
}

/*******************************************************************
* Function  : DumpPage
* Parameters: page_name, page_content
* Returns   : 
* Purpose   : If loggin is advanced, dumps page_content
			  to file <username>.<page_name>.html
*******************************************************************/
void CWebBrowser::DumpPage(const char* page_name, const char* page_content)
{
	if (settings.m_bCreateLog && settings.m_nLogLevel == LOG_ADVANCED)
	{            
		CStdString filename = username + "." + page_name + ".html";

		FILE *fp = fopen(LPCTSTR(filename), "w");

		if(fp != NULL)
		{
			fwrite(page_content, 1, strlen(page_content), fp);
			fclose(fp);               
			logFile.Write(LOG_ADVANCED, "<%s> Page has been dumped to '%s' for analysis", LPCTSTR(username), LPCTSTR(filename));
		}
		else
		{
			logFile.Write(LOG_ADVANCED, "<%s> Dump file '%s' could not be created.", LPCTSTR(username), LPCTSTR(filename));
		}
	}
}
