/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	WebBrowser.h
*
* Purpose : A simple class specifically designed to connect to
*           Yahoo! Mail and download emails.
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#ifndef _WEBBROWSER_H_
#define _WEBBROWSER_H_
#ifdef WIN32
#include "ProgressWnd.h"
#else
#include "Defines.h"
#endif

#define WEB_LOGIN_OK			0
#define WEB_LOGIN_FAILED		1
#define WEB_UNKNOWN_ERROR		2
#define WEB_WRONG_PASSWORD		3
#define WEB_WRONG_USERNAME		4
#define WEB_INBOX_FETCH_FAILED	5
#define WEB_INBOX_EMPTY         6
#define WEB_INBOX_FETCH_OK		0

#include "HttpSession.h"
#include "YahooStuff.h"
#include "Email.h"
#include "YahooSessionCache.h"

#ifdef WIN32
#define _WINSOCK2API_	// To ensure that MIME++ does not crib!
#endif
//#include <mimepp/mimepp.h>
#include "EmailMessage.h"

class CWebBrowser  
{
	typedef struct _msg_t
	{
		CStdString		mailbox;
		CStdString		msgId;
		Email			email;
		struct _msg_t	*next;
	} MESSAGE;

public:
	void EmptyBulkMail();
	CWebBrowser();
	virtual ~CWebBrowser();

	int			ConnectToYahoo(char *user, char *password);
	int			FetchMailboxList(const CStdString &folderName);
	int			GetMsgCount();
#ifdef WIN32
	int			DownloadAllEmails(CProgressWnd *progress);
#else
	int			DownloadAllEmails();
#endif
	int			DownloadEmail(MESSAGE *msg, BOOL headersonly=FALSE);
	int			DownloadEmail(int msgNumber, BOOL headersonly=FALSE);
    bool        AnyEmailsToDelete();
	void		DeleteEmails(const CStdString &folder);
	void		MarkUnread(const CStdString &folder);
	void		EmptyTrash();
	Email*		GetEmail(int msgNumber);
	void		SetSession(YAHOO_SESSION *ySession);
	void		SaveSession();
    void        DeleteSession();
    int         SendEmail(CStdString &to, CStdString &cc, 
						   CStdString &bcc, CStdString &subject, 
						   CStdString &body, bool htmlbody,
						   const EmailAttachmentList &attachment);

protected:
	CStdString  UrlEncode(const char *);
	void DumpPage(const char* page_name, const char* page_content);
	inline void rhex(unsigned int num, char *data);
		
	HttpSession *session;
	int			msgCount;
	int			downloadedCount;
	
	CStdString  host;
	CStdString  crumbValue;
	CStdString  username;
	CStdString  password;

	//*nix will need something too
#ifdef WIN32
	static CCriticalSection csConnect;
#else
	pthread_mutex_t csConnect;
#endif	
	MESSAGE *msgHead;
	MESSAGE *msgTail;
};

#endif
