/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	YahooSessionCache.cpp
*
* Purpose : Class for caching session cookies
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifndef WIN32
#include <time.h>
#endif

#include "YAHOOPOPS.h"
#include "YahooSessionCache.h"
#include "DataHolder.h"
#include "LogFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern LogFile logFile;
extern CDataHolder settings;

YahooSessionCache::YahooSessionCache()
{
	head = NULL;
	tail = NULL;
}

YahooSessionCache::~YahooSessionCache()
{
	YAHOO_SESSION *ptr = head;
	YAHOO_SESSION *temp;

	while(ptr != NULL)
	{
		
		delete ptr->session;
		temp = ptr->next;
		delete ptr;
		ptr = temp;
	}
}


//EDIT: Shane.H : Include the password in the session information
void YahooSessionCache::AddSession(const CStdString &username, const CStdString &password, const CStdString &crumbValue, HttpSession *session, const CStdString &host)
{
	YAHOO_SESSION *ptr = head;
	bool bFound = false;

	while(ptr != NULL && !bFound)
	{
		if(ptr->username == username && ptr->password == password)
		{
			bFound = true;
			break;
		}

		ptr = ptr->next;
	}

	if(bFound)
	{
		/* A session for this user exists */
		logFile.Write(LOG_ADVANCED, "<%s> Warning: session re-added", LPCTSTR(ptr->username));
		//ptr->loginTime = time(&ptr->loginTime);
	}
	else
	{
		YAHOO_SESSION *newSession = new YAHOO_SESSION;

		newSession->next = NULL;
		newSession->session = session;
		newSession->username = username;
		newSession->password = password;
		newSession->crumbValue = crumbValue;
		newSession->loginTime = time(&newSession->loginTime);
		newSession->host = host;

		if(head == NULL)
		{
			head = newSession;
			tail = newSession;
		}
		else
		{
			tail->next = newSession;
			tail = newSession;
		}
	}
}


//EDIT: Shane .H : Overloaded GetSession which takes a password and flag on whether to check it or not
YAHOO_SESSION* YahooSessionCache::GetSession(CStdString username, CStdString password, bool checkPass)
{
	YAHOO_SESSION *ptr = head;

	//Nows a good time to loop through and discard old sessions.
	while(ptr != NULL)
	{
		/* Check last access time (is actually creation time) */
			time_t now = time(NULL);
		//Yahoo cookies are valid for 8 hrs
		if(difftime(now, ptr->loginTime) > settings.m_nCookieTimeout)
			{
			logFile.Write(LOG_ADVANCED, "Session expired for %s", LPCTSTR(ptr->username));
			YAHOO_SESSION *temp=ptr->next;
			DeleteSession(ptr->username);
			ptr=temp;
		} else ptr = ptr->next;
			}

	ptr = head;
	while(ptr != NULL)
	{
		if(ptr->username == username && (ptr->password == password || checkPass == false))
		{
			/* Edit SH. Don't update the cookie last use value - yahoo expires them after 8 hours
			 */
			//ptr->loginTime = time(&ptr->loginTime);
			logFile.Write(LOG_ADVANCED, "<%s> Previous session recycled", LPCTSTR(username));
			return ptr;
		}
		ptr = ptr->next;
	}
	return NULL;
}

YAHOO_SESSION* YahooSessionCache::GetSession(CStdString username)
{
	return GetSession(username, "", false);
}

YAHOO_SESSION* YahooSessionCache::GetSession(CStdString username, CStdString password)
{
	return GetSession(username, password, true);
}

void YahooSessionCache::DeleteSession(CStdString username)
{
	YAHOO_SESSION *ptr = head;
	YAHOO_SESSION *temp = ptr;

	while(ptr != NULL)
	{
		if(ptr->username == username)
		{
			delete ptr->session;

			if(ptr == head)
			{
				temp = head->next;
				delete ptr;
				head = temp;
			}
			else if(ptr == tail)
			{
				delete ptr;
				temp->next = NULL;
				tail = temp;
			}
			else
			{
				temp->next = ptr->next;
				delete ptr;
			}

			break;
		}

		temp = ptr;
		ptr = ptr->next;
	}
	
	logFile.Write(LOG_ADVANCED, "<%s> Session deleted from cache", LPCTSTR(username));
}

