/*************************************************************************
*
* Name    :	vsscanf.h
*
* Author  : Anuj Seth <anujseth@yahoo.com>
*			K.Shyam <shyamk@mailcity.com>	
*
* Purpose : A Linux-like vsscanf implementation
*
*		    VC++ provides the vsprintf() function but for some strange
*           reason does not provide the complementary vsscanf() function
*           which is available on most Unix systems. Several C++
*           implementations exist to plug this hole but most of them do not
*           provide the identical API available on Linux. This is an
*           implementation which enhances the CStringEx::Scanf extension
*           written by Rainer Bauer to provide an identical API to that
*           which is available on Linux.
*--------------------------------------------------------------------------
* Copyright (c) 2002, Anuj Seth and Rainer Bauer and K.Shyam too :-). 
*--------------------------------------------------------------------------
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
* 2. All advertising materials mentioning features or use of this
*    software must display the following acknowledgment:
*    "This product includes software developed by Anuj Seth and Rainer
*    Bauer"
*
* 3. Redistributions of any form whatsoever must retain the following
*    acknowledgment:
*    "This product includes software developed by Anuj Seth and Rainer
*    Bauer"
*
*-------------------------------------------------------------------------
* This code is based on Rainer Bauer's CStringEx::Scanf extension class
* Website: http://codeguru.earthweb.com/string/StringExScanf.html
* CStringEx::Scanf extension Copyright (c) Rainer Bauer, 2001
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#ifndef _VSSCANF_H_
#define _VSSCANF_H_

/* modified by K.Shyam */
#ifdef WIN32
#include "stdafx.h"
#define STSCANF _stscanf
#define TCSCHR  _tcschr
#else
/* now for some windows data types ! */
#include <stdarg.h>
#include <iostream.h>
#include <string.h>
#include "StdString.h"
#include "Defines.h"
#define STSCANF sscanf
#define TCSCHR  strchr
/* phew ! finally done ! */
#endif

int ourvsscanf( const char *str, LPCTSTR format, va_list arglist);

#endif // #ifndef _VSSCANF_H_

